<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\FPA\Services\RelatCustomIntegraFinanService;

class RelatCustomIntegraFinanController extends Crud {
    /** @var RelatCustomIntegraFinanService */
    protected $integraFinanService;

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (
        Manager $dataSourceManager,
        ParameterBag $parameterBag,
        HCMParameters $hcmParameters,
        EntityManager $entityManager,
        RelatCustomIntegraFinanService $integraFinanService
    ){
        $this->parameterBag       = $parameterBag;
        $this->hcmParameters      = $hcmParameters;
        $this->entityManager      = $entityManager;
        $this->pessoa             = Repositories::GPE_PESSOA;
        $this->integraFinanService   = $integraFinanService;
        parent::__construct($dataSourceManager);
    }
    
    
    public function getRelatCustomIntegraFinan(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $row = $request->getFilterCriteria();
            $newFilterData = $this->populateParameterBag($row);
            $arrayFilter = Util::filtroToArray($newFilterData);
            $arrayReturned = $this->columnToUpper($arrayFilter);
            $arrayReturned = $this->prepareFilterRow($arrayReturned);
            $dataset = $this->integraFinanService->getRelatCustomIntegraFinan($arrayReturned);
            
            if(key($dataset) == 'messages') {
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataset['messages'][0]['message']);
                $response->addMessage($message);
                $response->addDataSet(new DataSet('data', []));
            } else {
               $preparedDataset = $this->prepareDatasetRow($arrayReturned, $dataset) ?? [];
               $response->addDataSet(new DataSet('data', $preparedDataset));
            }

            
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    protected function prepareDatasetRow($arrayReturned, $dataset): array {
        $toReturn = array();
        $pagamentos = $dataset['dataset']['pagamentos'] ?? [];

        foreach($pagamentos as $dataInPagamentos) {
            $dataAuxiliar = !empty($dataInPagamentos['data']) ? DateUtil::getDataDeString($dataInPagamentos['data'], 'Y-m-d H:i:s', true) : null;
            if (!empty($dataAuxiliar)) {
                $dataInPagamentos['data'] = $dataAuxiliar->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }
            $toReturn[] = $dataInPagamentos;
        }
        
        return $toReturn;
    }
    
    protected function columnToUpper($arrayFilter): array {
        $toReturn = array();

        foreach($arrayFilter as $key => $value) {
            $toReturn[strtoupper($key)] = $value;
        }

        return $toReturn;
    }
    
    protected function prepareFilterRow($arrayFilterRow): array {
        $arrayFilterRow['CNPJ'] = preg_replace("/[^0-9]/", "", $arrayFilterRow['CNPJ']);
        if(!empty($arrayFilterRow['CNPJFORNECEDOR'])){
            $arrayFilterRow['CNPJFORNECEDOR'] = preg_replace("/[^0-9]/", "", $arrayFilterRow['CNPJFORNECEDOR']);
        }
        $arrayFilterRow['DATACOMPETENCIA'] = $arrayFilterRow['DATACOMPETENCIA'] ?? $this->hcmParameters->getDtCompetencia();
        return $arrayFilterRow;
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
} 