<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\FPA\Repository\CalculoFolhaPercRepository;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;

class CalculoFolhaPercService {

    private $hcmParameters;
    private $calculoFolhaPercRepository;
    private $vinculomEntity;

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_dtCompetencia;
    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_dtPeriodIni;
    private $_dtPeriodFim;
    private $_eventosM;
    private $_nrOcorreCal;
    private $_nrTipoMoviment;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, CalculoFolhaPercRepository $CalculoFolhaPercRepository, LoteLancEventoService $loteLancEventoService){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->calculoFolhaPercRepository = $CalculoFolhaPercRepository;
        $this->loteLancEventoService  = $loteLancEventoService;
        $this->vinculomEntity = new GpevinculomEntity($entityManager, $hcmParameters);
    }

    private function _inicializaVariaveis($dataSet) {
        $this->_nrOrg           = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao     = $this->hcmParameters->getNrorgPadrao();
        $this->_cdOperador      = $this->hcmParameters->getCdOperador();
        $this->_dtCompetencia   = $this->hcmParameters->getDtCompetencia();
        $this->_dsInicio = $dataSet['NOMEINICIAL'];
        $this->_dsFim    = $dataSet['NOMEFINAL'];
        $this->_dsOrdem  = $dataSet['DSORDEM'];
        $this->_eventosM = $dataSet['NREVENTOM'];
        $this->_vrPerc   = $dataSet['VRPERC'];
        $this->_nrOcorreCal    = empty($dataSet['NROCORRECAL']) ? NULL : $dataSet['NROCORRECAL'];
        $this->_nrTipoMoviment = $dataSet['NRTIPOMOVIMENT'];
        $this->_nmTipoMoviment = $dataSet['NMTIPOMOVIMENT'];
        $this->_dtPagamento = substr($dataSet['DTPAGAMENTO'],0, 10);
        $this->_dtPeriodIni = substr($dataSet['DTPERIODINI'],0, 10);
        $this->_dtPeriodFim = substr($dataSet['DTPERIODFIM'],0, 10);
    }
    
    public function calculoPerc($dataSet){
        try{
            
            $this->_inicializaVariaveis($dataSet);
            
            $vinculos = $this->vinculomEntity->retornaVinculosAtivosMesCompetencia(
                $this->_nrOrg, $this->_dtCompetencia, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem,
                false, $nrEstruturaTrab, true, false, null,  null, null, null, false, null);

            if(count($vinculos) > 0) {
                $itensLote = [];

                foreach($vinculos as $vinculo) {
                    foreach($this->_eventosM as $evento) {

                        $params = ['P_NRORG'          => $this->_nrOrg,
                                   'P_NREVENTOM'      => $evento,
                                   'P_NRTIPOMOVIMENT' => $this->_nrTipoMoviment,
                                   'P_NROCOR'         => $this->_nrOcorreCal,
                                   'P_NRVINCULOM'     => $vinculo->getNrvinculom(),
                                   'P_NRTPMODALIDCAL' => 1,
                                   'P_INI'            => $this->_dtPeriodIni,
                                   'P_FIN'            => $this->_dtPeriodFim];

                        $vrEventoTotal = $this->calculoFolhaPercRepository->getSumEventos($params);

                        if($vrEventoTotal) {
                            $itensLote[] = ['NRVINCULOM'     => $vinculo->getNrvinculom(),
                                            'NREVENTOM'      => $evento,
                                            'DTOCORRENCIA'   => $this->_dtPeriodIni,
                                            'VRREFFOLHALOTE' => null,
                                            'VREVENTOLOTE'   => $vrEventoTotal['VREVENTOFOLHA'] * ($this->_vrPerc / 100)];
                        }
                    }
                }

                if($itensLote) {
                    $obs = "Período Inicial: ".$this->_dtPeriodIni."\n".
                           "Período Final: ".$this->_dtPeriodFim."\n".
                           "Percentual: ".$this->_vrPerc."\n".
                           "Tipo de Movimento: ".$this->_nrTipoMoviment."-".$this->_nmTipoMoviment."\n".
                           "Critério de Seleção: ".$this->_dsOrdem;

                    $lote = $this->loteLancEventoService->geraLoteLancEvento(
                        $this->_nrOrg, $this->_dtCompetencia, 'FPA', 'EXPORTAÇÃO DO CÁLCULO DE DIFERENÇA DISSÍDIO - PERCENTUAL', 'PENDENTE', $obs, $this->_nrTipoMoviment, $itensLote);

                    return 'Processado com sucesso o lote nº '.$lote;
                } else {
                    throw new \Exception("Nenhum item de lote foi gerado. Verifique os critérios utilizados.");
                }
            } else {
                return false;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
}
