<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 02/08/2016
 * Time: 18:15
 */

namespace HCM\FPA\Services;

use Folha\Entities\FpaItecalcfolha;
use Folha\Entities\FpaTipoassoceve;
use Folha\Entities\FpaTipomoviment;
use Folha\Entities\GpeVinculom;
use HCM\FPA\Repository\FpaitecalcfolhaRepository;
use HCM\FPA\Repository\BloqueioEventoVinculoRepository;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use Doctrine\ORM\EntityManager;


class ItensCalculoService {

    protected $hcmParameters;

    public function __construct(HCMParameters $hcmParameters, FpaitecalcfolhaRepository $fpaitecalcfolhaRepository, BloqueioEventoVinculoRepository $bloqueioVinculoRepository, EntityManager $entityManager) {
        $this->hcmParameters                = $hcmParameters;
        $this->fpaitecalcfolhaRepository    = $fpaitecalcfolhaRepository;
        $this->FpaItecalcfolha              = Repositories::FPA_ITECALCFOLHA;
        $this->GpeVinculom                  = Repositories::GPE_VINCULOM;
        $this->bloqueioEventVincRepository  = $bloqueioVinculoRepository;
        $this->entityManager                = $entityManager;
    }

    /**
     * Verifica se o vínculo possui líquido no cálculo em questão.
     *
     * Em caso de movimento mensal, primeiro verifica se existe líquido de rescisão e retorna caso exista.
     * Caso não exista verifica líquido normal assim como para outros movimentos.
     *
     * @param $nrtipomoviment
     * @param $nrcalculofolha
     * @param $nrvinculom
     * @param $nrorg
     * @return bool
     */
    public function validaLiquido($nrtipomoviment, $nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg)
    {
        try {
            if ($nrtipomoviment == FpaTipomoviment::TP_MOV_MENSAL) {
                $liquido = FpaItecalcfolha::validaLiquido($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, FpaTipoassoceve::ASSOC_LIQUIDO_RESCISAO);
                if ($liquido) {
                    return $liquido;
                }
            }
            return FpaItecalcfolha::validaLiquido($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, FpaTipoassoceve::ASSOC_LIQUIDO);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLiquidoBeneficio($nrtipomoviment, $nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, $nreventom, $dtpagamento)
    {
        try {
            return FpaItecalcfolha::validaLiquidoBeneficio($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, $nreventom, $dtpagamento );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLiquidoBeneficioSemDataPagamento($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, $nreventom)
    {
        try {
            return FpaItecalcfolha::validaLiquidoBeneficioSemDataPagamento($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, $nreventom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getIdeventodemonst($nrorg_padrao, $nreventom, $dtCompetencia, $nrorg)
    {
        try {
            return $this->fpaitecalcfolhaRepository->getIdeventodemonst($nrorg_padrao, $nreventom, $dtCompetencia, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaValorReferenciaBloqueiaEdicao()
    {
        try {
            return $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), 0, $this->hcmParameters->getDtCompetencia(), 'BLOQUEIA_EDICAO_EVENTO_TOTAL');
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaValorReferenciaBloqueiaEncargo()
    {
        try {
            return $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), 0, $this->hcmParameters->getDtCompetencia(), 'BLOQUEIA_EDICAO_EVENTO_ENCARGO');
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaValorReferenciaBloqueiaComplementoLiquido()
    {
        try {
            return $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), 0, $this->hcmParameters->getDtCompetencia(), 'BLOQUEIA_EDICAO_COMPL_LIQUIDO');
        } catch (\Exception $e) {
            throw $e;
        }
    }
   
    public function retornaItemCalculo($nrorg, $nrcalculofolha, $nreventom, $nrvinculom, $dtocorrencia)
    {
        try {
            return $this->fpaitecalcfolhaRepository->getUniqueItem($nrorg, $nrcalculofolha, $nreventom, $nrvinculom, $dtocorrencia);
        } catch (\Exception $e) {
            throw $e;
        }
    }
      
    public function updateItensCalc($row){
        try {
            
            $iteCalc = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy(array('nrorg' => $row['NRORG'],
            'nrvinculom' => $row['NRVINCULOM'], 'nritemcalcfolha' => $row['NRITEMCALCFOLHA']));
            
            $idRefeValorCalc = $row['IDREFEVALORCALC'];
            
            if($row['IDREFEVALORCALC'] == 'NORMAL' || !$row['IDREFEVALORCALC']){
                if((isset($row['VREVENTOFOLHA_inline']) && (isset($row['changed_VREVENTOFOLHA']) &&  ($row['changed_VREVENTOFOLHA'] != $row["VREVENTOFOLHA_inline"]))) && (isset($row['VRREFFOLHA_inline']) && (isset($row['changed_VRREFFOLHA']) && ($row['changed_VRREFFOLHA'] != $row["VRREFFOLHA_inline"])))){
                    $idRefeValorCalc = 'REFERENCIA_VALOR';
                }else if(isset($row['VREVENTOFOLHA_inline']) && (isset($row['changed_VREVENTOFOLHA']) && ($row['changed_VREVENTOFOLHA'] != $row["VREVENTOFOLHA_inline"]))){
                    $idRefeValorCalc = 'VALOR';
                }else if(isset($row['VRREFFOLHA_inline']) && (isset($row['changed_VRREFFOLHA']) && ($row['changed_VRREFFOLHA'] != $row["VRREFFOLHA_inline"]))){
                    $idRefeValorCalc = 'REFERENCIA';
                }
            }
            
            $iteCalc->setVreventofolha($row['VREVENTOFOLHA']);
            $iteCalc->setVrreffolha($row['VRREFFOLHA']);
            $iteCalc->setIdrefevalorcalc($idRefeValorCalc);
            
            \Setforms2\Shared\Registry::getEntityManager()->persist($iteCalc);
            \Setforms2\Shared\Registry::getEntityManager()->flush($iteCalc);
            
        } catch (Exception $e) {
            throw $e;    
        }
    }
    
    
    public function verificaEventoBloqueado ($nrorg, $nrevento, $tipoVinculo) {
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrorg_padrao = $this->hcmParameters->getNrorgPadrao();
        return $this->bloqueioEventVincRepository->retornaEventosBloqueados($nrorg, $nrorg_padrao, $dtmescompetenc, $nrevento, $tipoVinculo);
    }
    

    public function validaVinculos($row, $vinculosRescindidos){
        
        $arrayvinc = $vinculosRescindidos; 
        
        foreach($vinculosRescindidos as $arrayvinc){
        
            $nrvinculom = $arrayvinc->getNrvinculom();
            
            if($nrvinculom){
                $this->insereEvento($nrvinculom, $row);
            }
        }
    }
    
    
    public function insereEvento($nrvinculom, $row){
        $nreventos = [3010,3020,3050];
        foreach($nreventos as & $nreventom){
             try{
                $this->entityManager->beginTransaction();
                $e3010 = new $this->FpaItecalcfolha;
                $e3010->setNrorg($row['NRORG']);
                $e3010->setNrvinculom($nrvinculom);
                $e3010->setVrreffolha($row['VRREFFOLHA']);
                $e3010->setVreventofolha($row['VREVENTOFOLHA']);
                $e3010->setNritemcalcfolha(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 12, true));
                $e3010->setNrcalculofolha($row['NRCALCULOFOLHA']);
                $e3010->setNreventom($nreventom);
                $e3010->setDtocorrencia(DateUtil::getDataDeString($row['DTPAGAMENTO']));
                $e3010->setIdrefevalorcalc('NORMAL');
                $e3010->setQtocorproxperi(1);
                
                
                // set auditoria
                $e3010->setDtinclusao(DateUtil::getDataAtual());
                $e3010->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $e3010->setNrorginclusao($row['NRORG']);
                $e3010->setDtultatu(DateUtil::getDataAtual());
                $e3010->setCdoperultatu($this->hcmParameters->getCdoperador());
                $e3010->setNrorgultatu($row['NRORG']);
                $this->entityManager->persist($e3010);
                $this->entityManager->flush($e3010);
                $this->entityManager->commit();
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
        }
        
    }

    public function deleteItens($itensToDelete)
    {
      $this->entityManager->beginTransaction();

      $query = $this->entityManager->getConnection()->prepare("
        DELETE FROM FPA_ITECALCFOLHA
        WHERE NRORG = :P_NRORG AND NRITEMCALCFOLHA IN (".implode(', ', $itensToDelete).")
      ");

      $query->execute(
        array(
          "P_NRORG" => $this->hcmParameters->getNrorgTrabalho()
        )
      );

      $this->entityManager->flush();
      $this->entityManager->commit();
    }
    
    
   
}