<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Repository\ResumoContabilRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\Util\DateUtil;
use HCM\Util\FPA;


class ResumoContabilService {
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ResumoContabilRepository $resumoContabilRepository, RelatoriosHcmController $relatoriosHcmController) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->resumoContabilRepository = $resumoContabilRepository;
        $this->relatoriosHcmController = $relatoriosHcmController;
    }

    public function geraArquivoLotePadrao($dtocorrencia, $nrorg, $cdoperador, $response, $dtcontabil, $nrtipomoviment = null, $listaestrutsup = null, $reversao, $unificararq)
    {
        try {
            $filtro = ['P_NRORG' => $nrorg,
                       'P_OCORRENCIA' => $dtocorrencia,
                       'P_NRTIPOMOVIMENT' => $nrtipomoviment,
                       'P_COMPETENCIA'=>$this->hcmParameters->getDtCompetencia(),
                       'download' => ['format' => "txt",
                                      'strFind' => "·",
                                      'strReplace' => "\r"]];
            if($dtcontabil)
                $filtro['P_DTCONTABIL'] = $dtcontabil;
            
            $estruturas = $this->resumoContabilRepository->getEstruturasSupResCont($dtocorrencia, $nrorg, $nrtipomoviment, $listaestrutsup);

            if($unificararq == 'N') {
                foreach ($estruturas as &$estrutura) {
                    //$nrestrutsuperior = $hierarquia_org->determinaEstruturaSuperior($estrutura['NRESTRUTURAM'], $hierarquias);
        
                    if (is_null($estrutura['NRESTRUTURAMSUP'])) {
                        $message = "Existem resumos sem a estrutura superior. Verifique a hierarquia e realize a contabilização novamente.";
                        $nrEstrutura = null;
                        $nmEstrutura = 'SEM_ESTRUTURA';
        
                        $nome_arquivo = 'integracao_resumo_contabil_'.$nmEstrutura;
                    }
                    else {
                        $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
                            array(
                                'nrestruturam' => $estrutura['NRESTRUTURAMSUP'],
                                'nrorg' => $nrorg
                            )
                        );
                        $nrEstrutura = $estruturam->getNrestruturam();
                        $nmEstrutura = $estruturam->getNmestruturam();
        
                        $nome_arquivo = 'integracao_resumo_contabil_'.$nrEstrutura.'-'.$nmEstrutura;
                    }
        
                    $nome_arquivo = str_replace( array(' ','\\','/',':','*','?','"','<','>','|','&'), '', $nome_arquivo);

                    $filtro['P_ESTRUTURA'] = $nrEstrutura;
                    $filtro['download']['fileName'] = $nome_arquivo;
                
                    if($reversao == 'N') {
                        $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'integracao_resumo_contabil', null, $response);
                    } else {
                        $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'integracao_resumo_contabil_reverso', null, $response); 
                    }
                }
            } else {
                $arrEstruturas = [];

                foreach ($estruturas as $estrutura) {
                    if(!is_null($estrutura['NRESTRUTURAMSUP']))
                        $arrEstruturas[] = $estrutura['NRESTRUTURAMSUP'];
                }
                
                if($arrEstruturas) {
                    $arrEstruturas = implode(',', $arrEstruturas);
                    $nrsEstrutura  = str_replace( [','], '_', $arrEstruturas);
                    $nome_arquivo  = 'integracao_resumo_contabil_unificado_'.$nrsEstrutura;
                    $nome_arquivo  = str_replace( [' ','\\','/',':','*','?','"','<','>','|','&'], '', $nome_arquivo);
                    $filtro['P_ESTRUTURA'] = $arrEstruturas;
                    $filtro['download']['fileName'] = $nome_arquivo;

                    $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'integracao_resumo_contabil_unificado', null, $response); 
                }
                else $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Existem resumos sem a estrutura superior. Verifique a hierarquia e realize a contabilização novamente.<br>", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));   
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraArquivoProvisaoFerias($nrorg, $nrEstrutLegal, $dtContabil)
    {
        try {
            $name = 'Provisao_Ferias';
            $resumos = $this->resumoContabilRepository->getProvisaoFerias($nrorg, $nrEstrutLegal, $dtContabil);

            if(empty($resumos)) {
                return null;
            }

            $escopoGeral = 'Empresa;Competencia;Origem;Tipo;Local;Base;Dias;Anterior;Transferido;Ajuste;Provisao;Pagas;Adiantado;Indenizado;Saldo;';

            foreach ($resumos as $resumo) {
                $escopo = $escopoGeral.PHP_EOL.$resumo['REGISTRO'];
                $escopoGeral = $escopo;
            }

            $arquivo = $this->salvarMsgArquivo($escopoGeral, $name);

            return $arquivo;
        
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraArquivoProvisao13($nrorg, $nrEstrutLegal, $dtContabil)
    {
        try {
            $name = 'Provisao_13';
            $resumos = $this->resumoContabilRepository->getProvisao13($nrorg, $nrEstrutLegal, $dtContabil);

            if(empty($resumos)) {
                return null;
            }

            $escopoGeral = 'Empresa;Competencia;Origem;Tipo;Local;Base;Dias;Anterior;Transferido;Ajuste;Provisao;Pagas;Adiantado;Indenizado;Saldo;';

            foreach ($resumos as $resumo) {
                $escopo = $escopoGeral.PHP_EOL.$resumo['REGISTRO'];
                $escopoGeral = $escopo;
            }

            $arquivo = $this->salvarMsgArquivo($escopoGeral, $name);

            return $arquivo;
        
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraArquivoFolha13Ferias($nrorg, $nrEstrutLegal, $dtContabil)
    {
        try {
            $name = 'Folha_13_Ferias';
            $resumos = $this->resumoContabilRepository->getFolha13Ferias($nrorg, $nrEstrutLegal, $dtContabil);

            if(empty($resumos)) {
                return null;
            }

            $escopoGeral = 'Indicador folha % Data de lançamento % Total de débito % Total de crédito % Total dos lançamento % Histórico Padrão % Constante % Líquido dos Lançamentos';

            foreach ($resumos as $resumo) {
                $escopo = $escopoGeral.PHP_EOL.$resumo['REGISTRO'];
                $escopoGeral = $escopo;
            }

            $arquivo = $this->salvarMsgArquivo($escopoGeral, $name);

            return $arquivo;
        
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraArquivoFolha($nrorg, $nrEstrutLegal, $dtContabil)
    {
        try {
            $name = 'Folha';
            $resumos = $this->resumoContabilRepository->getFolha($nrorg, $nrEstrutLegal, $dtContabil);

            if(empty($resumos)) {
                return null;
            }

            $escopoGeral = 'Código Registro;Data;Local;Constante;Constante;Total de Valores;Constante;Nome Relatório;Filial;Evento
Código Registro;Constante;Local;Total de Valores;Constante;Constante;Evento';

            foreach ($resumos as $resumo) {
                $escopo = $escopoGeral.PHP_EOL.$resumo['REGISTRO'];
                $escopoGeral = $escopo;
            }

            $arquivo = $this->salvarMsgArquivo($escopoGeral, $name);

            return $arquivo;
        
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarMsgArquivo($escopoGeral, $name){
        $arquivo = FPA::getTempDirectory().'/'.$name.' Execucao('.DateUtil::getDataAtual()->format('d-m-Y H:i:s').').txt';
        file_put_contents($arquivo, $escopoGeral);
        return $arquivo;
    }

} 
