<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2190Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS2190($params) {
        $sql = <<<SQL
SELECT VM.NRVINCULOM as NRCHAVE,
       VM.NRVINCULOM,
       '' as NRTABELA,
       'I' as IDOPERACAO,
       PH.DTADMISSAOPRE as DTMESCOMPETENC
  FROM GPE_PESSOA PE
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN GPE_VINCULOM VM ON VM.NRORG = PE.NRORG
                      AND VM.NRPESSOA = PE.NRPESSOA
  JOIN GPE_VINCULOH VH ON VH.NRORG = PE.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = '01/01/3000'
 WHERE PE.NRORG = :P_NRORG
   AND PH.DTADMISSAOPRE IS NOT NULL
   AND NOT EXISTS (SELECT 1
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-2190'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                     ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
                    WHERE EC.NRORG = PE.NRORG
                      AND EC.NRCHAVE = PE.NRPESSOA
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= PH.DTMESCOMPETENC)
   AND (PE.NRPESSOA = :P_NRPESSOA OR :P_NRPESSOA IS NULL)
   AND (PH.DTADMISSAOPRE >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
 ORDER BY 1,4
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRPESSOA" => isset($params['NRPESSOA']) ? $params['NRPESSOA'] : null,
            "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }

    public function getDadosS2190($parametros) {
        try {
            return $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRVINCULOM'], $parametros['NRESTRUTURAM']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrenvio, $nrchave, $nrvinculom, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2190*/
SELECT PE.NRPESSOA CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,   
       --infoRegPrelim
       REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS CPFTRAB,
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') AS dtNascto,
       TO_CHAR(PH.DTADMISSAOPRE,'YYYY-MM-DD') AS dtAdm,
       VM.CDMATRICULA AS MATRICULA,
       CE.CDESOCIAL AS CODCATEG,
       CASE 
        WHEN CE.CDESOCIAL IN ('401','731','734','738') 
            THEN (CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END)
        WHEN CE.CDESOCIAL IN ('104') 
            THEN '1' 
        WHEN CE.CDESOCIAL IN ('102') 
            THEN '2'
        WHEN CE.CDESOCIAL IN ('721', '722', '771', '901')
            THEN ''
        ELSE (CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END)
        END natAtividade,
      --infoRegCTPS
       CH.NRCBO CBOCARGO,
       DECODE(SL.IDTPSALARIO,'HORA', DECODE(SL.VRSALARIOHR, NULL, (SL.VRSALARIO / EH.QTHRESCALATRABH * 30), SL.VRSALARIOHR), SL.VRSALARIO) vrSalFx,
       DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') AS UNDSALFIXO,
       CASE WHEN VM.NRVINCULOEMPREG IN (9, 10, 11, 12, 13) THEN '2' ELSE '1' END tpContr,
       CASE WHEN VM.NRVINCULOEMPREG IN (9, 10, 11, 12, 13) THEN TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD') ELSE NULL END dtTerm
  FROM GPE_PESSOA PE
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_PESSOAH.DTMESCOMPETENC FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG
                               ORDER BY GPE_PESSOAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN GPE_VINCULOM VM ON VM.NRORG = PE.NRORG
                      AND VM.NRPESSOA = PE.NRPESSOA
  JOIN GPE_VINCULOH VH ON VH.NRORG = PE.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  LEFT JOIN GPE_OCUPACAOH CH ON CH.NROCUPACAOM = VH.NRCARGO
                            AND CH.NRORG = VH.NRORG
                            AND CH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                        AND GPE_OCUPACAOH.NROCUPACAOM = CH.NROCUPACAOM
                                                        AND GPE_OCUPACAOH.NRORG = CH.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT * FROM (SELECT ESTRUTURAH.DTMESCOMPETENC FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG
                                                 ORDER BY ESTRUTURAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG

  /*Horário*/
  LEFT JOIN GPE_ALTEESCALA AE ON AE.NRORG = VH.NRORG
                             AND AE.NRVINCULOM = VH.NRVINCULOM
                             AND (AE.DTFIMESCALA IS NULL OR AE.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC) OR VM.DTRESCISAOVINC IS NOT NULL)
                             AND AE.DTINIESCALA = (SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                                                    WHERE GPE_ALTEESCALA.DTINIESCALA <= LAST_DAY(:P_DTMESCOMPETENC)
                                                      AND (GPE_ALTEESCALA.DTFIMESCALA IS NULL OR GPE_ALTEESCALA.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC) OR VM.DTRESCISAOVINC IS NOT NULL)
                                                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                                                      AND GPE_ALTEESCALA.NRORG = AE.NRORG)
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = AE.NRESCALATRABM
                              AND EH.NRORG = AE.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
 WHERE PE.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRVINCULOM" => $nrvinculom,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
}