<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\PTO\Services\VinculoBancoHorasService;

class VinculoBancoHorasController extends Crud {

    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, VinculoBancoHorasService $vinculoBancoHorasService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->vinculoBancoHorasService = $vinculoBancoHorasService;
        parent::__construct($dataSourceManager);
    }

    public function getVinculoBancoHoras (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "GpeVinculom_getVinculoBancoHoras");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

     public function getAlterBancoHoras (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName = "getAlterBancoHoras");
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    public function setAlterBancoHoras (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{    
            $this->dataSourceName = "PtoAltebancohoras_setAlterBancoHoras";
            foreach($request->getDataSet()->getRows() as &$row) {
                $hasAlteBancoHorasAtivo = $this->vinculoBancoHorasService->analisaBancoHorasAtivo($row);
                if($hasAlteBancoHorasAtivo == true){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erroBancoHorasDtFimVazio', array('message' => "O vínculo: ".$row['NRVINCULOM']. " - ". $row['NMVINCULOM']. " já possui uma alteração de banco de horas ativa. Para que possamos prosseguir com o cadastro de um novo banco de horas para o vínculo, por favor, preencha o último registro de alteração, incluindo a data de término.")));
                    return;
                }
                if(array_key_exists('NRALTEBANCOHORAS', $row)){
                    if(empty($row['NRALTEBANCOHORAS'])){
                        $row['NRALTEBANCOHORAS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_ALTEBANCOHORAS', 1, 12, true);
                    }
                } else {
                    $row['NRALTEBANCOHORAS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_ALTEBANCOHORAS', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                        $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "PtoAltebancohoras_setAlterBancoHoras";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function cadastroAutomaticoAlterBancoHoras (DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            
            if (!isset($row['NRVINCULOM_LIST']) || !isset($row['NRBANCOHORAS']) || !isset($row['DTINICIOALTEBHORAS'])) {
                throw new \Exception('Os campos obrigatórios não foram enviados. Os campos obrigatórios são \'NRVINCULOM_LIST\', \'NRBANCOHORAS\' e \'DTINICIOALTEBHORAS\'');
            }
            
            // Obrigatórios
            $vinculos           = $row['NRVINCULOM_LIST'];
            $bancoHoras         = $row['NRBANCOHORAS'];
            $dtinicioaltebhoras = $row['DTINICIOALTEBHORAS'];
            
            // Opcional
            $dtfimaltebhoras    = $row['DTFIMALTEBHORAS'] ?? null;
            
            $cadastroAutomaticoOk = $this->vinculoBancoHorasService->cadastroAutomaticoAlterBancoHoras($vinculos, $bancoHoras, $dtinicioaltebhoras, $dtfimaltebhoras);
            
            $result = array('success' => $cadastroAutomaticoOk);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cadastroAutomaticoAlterBancoHoras', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}