<?php

namespace HCM\PTO\Logic\AFD\Portaria671;

/**
 * Portaria 671
 * Gera valores de validacao sobre os itens de registros
 * 
 */

use HCM\Util\Util;
use HCM\PTO\Logic\AFD\ValidadorRegistro as ValidadorRegistroAFD;

class ValidadorRegistroSHA256 extends ValidadorRegistroAFD {
    /**
     * Portaria 671
     * Inclui a validacao CRC-16 (Cyclic Redundancy Check) para cada item
     * @param array $dadosRegistro
     * @return array
     */
    public function incluiValidacaoItens(array $dadosRegistro): array {
        $ultimaHash = null;
        foreach ($dadosRegistro as &$dadoRegistro) {
            // Gera a hash utilizando as informacoes da linha e a hash do ultimo registro
            $hash = substr(str_pad(hash('sha256', $dadoRegistro . $ultimaHash), 64), 0, 64);
            $dadoRegistro .= $hash;
            $ultimaHash = $hash;
        }
        return $dadosRegistro;
    }
}