<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request\DataSet;
use HCM\SST\Services\InstrumentosAvalRiscosService;

class InstrumentosAvalRiscosController extends Crud {

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRINSTRUMENTO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, InstrumentosAvalRiscosService $instrumentosAvalRiscosService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->instrumentosAvalRiscosService = $instrumentosAvalRiscosService;
        parent::__construct($dataSourceManager);
    }

    public function getInstrumentosAvalRiscos(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "sst_instrumentosavalriscos_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setInstrumento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "sst_instrumentosavalriscos";
            foreach ($request->getDataSet()->getRows() as &$row) {
               $row['IDATIVO'] = $row['IDATIVO']?? 'S';
               $instrumentoExistente = $this->instrumentosAvalRiscosService->existeInstrumento($row['NRORG'], $row['NMINSTRUMENTO']);

               if($instrumentoExistente && ($instrumentoExistente['NRINSTRUMENTO'] != $row['NRINSTRUMENTO'] || !isset($row['NRINSTRUMENTO'])) ) {
                    $response->addNotification(new Notification("Já existe um Instrumento com esse nome", Notification::TYPE_ALERT));
                    return;
                }  

                if($row['__is_new'] === true){
                    $row['NRINSTRUMENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_INSTRUMENTO', 1, 12, true); 
                }
               
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteInstrumento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "sst_instrumentosavalriscos";
            $nrorg = $request->getDataSet()->getRows()[0]['NRORG'];
            $nrinstrumento = $request->getDataSet()->getRows()[0]['NRINSTRUMENTO'];

            $existeItens = $this->instrumentosAvalRiscosService->existeItensFilhos($nrorg, $nrinstrumento);

            if(!$existeItens){
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem itens relacionados a este registro, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function getMarcaInstrumento(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "sst_marcainstrumento_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setMarcaInstrumento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "sst_marcainstrumento";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';

                foreach($row['NRMARCA_LIST'] as $nrmarca) {
                    $marcaInstrumentoExistente = $this->instrumentosAvalRiscosService->existeMarcaInstrumento($row['NRORG'], $nrmarca, $row['NRINSTRUMENTO']);

                    if(!$marcaInstrumentoExistente) {
                        $row['NRMARCA'] = $nrmarca;
                        if($row['__is_new'] === true){
                            $row['NRMARCAINSTRUMENTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'SST_MARCAINSTRUMENTO', 1, 12, true); 
                        }
            
                        parent::save($request, $response);
                    }
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteMarcaInstrumento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "sst_marcainstrumento";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function setMarca(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "sst_marca";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';

                $marcaExistente = $this->instrumentosAvalRiscosService->existeMarca($row);

                if( !$marcaExistente ) {
                    $row['NRMARCA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'SST_MARCA', 1, 12, true); 
                    parent::save($request, $response);
                } else {
                    $response->addNotification(new Notification("Já existe uma marca com esse nome", Notification::TYPE_ALERT));
                }

            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getMarca(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "sst_marca_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition)
    {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria)
    {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 