<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 29/06/2016
 * Time: 17:42
 */

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\RelestrutriscoService;

class RelestrutriscoController extends Crud {

    protected $dataSourceName = 'sst_relestrutrisco';
    protected $dataSourceName_get = 'sst_relestrutrisco_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, RelestrutriscoService $relestrutriscoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->relestrutriscoService = $relestrutriscoService;
        parent::__construct($dataSourceManager);
    }

    public function getRelestrutrisco (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRelestrutrisco (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRELESTRUTRISCO', $row)) {
                    if ($row['NRRELESTRUTRISCO'] === '') {
                        $row['NRRELESTRUTRISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RELESTRUTRISCO', 1, 12, true);
                    }
                } else {
                    $row['NRRELESTRUTRISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_RELESTRUTRISCO', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRelestrutrisco(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function cadAutFatorRiscoPorEstrutura(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach ($request->getDataSet()->getRows() as $row) {
                foreach($row['NRESTRUTURAM'] as $nrEstrutura){
                    foreach($row['NRFATORRISCO'] as $nrFatorRisco){
                        $fatorRiscoEstrut = $this->relestrutriscoService->getRelEstrutRiscoCad($row['NRORG'], $nrEstrutura, $nrFatorRisco);
                        if(!isset($fatorRiscoEstrut)){
                            $this->relestrutriscoService->cadAutFatorRiscoPorEstrutura($nrEstrutura, $nrFatorRisco, $row);
                        }
                    }
                }
            }
            $response->addNotification(new DTO\Response\Notification('cadAutoAssocCtbl'), DTO\Response\Notification::TYPE_SUCCESS);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 