<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\GpeReuniao;
use Folha\Entities\GpeEpi;
use Folha\Entities\GpeEpiparticipa;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\SST\Repository\EpiRepository;
use HCM\Util\DateUtil;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;
use HCM\Util\Repositories;

class EpiService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, EpiRepository $epiRepository, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel) {
        $this->entityManager          = $entityManager;
        $this->hcmParameters          = $hcmParameters;
        $this->epiRepository          = $epiRepository;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel             = $httpKernel;
        $this->sst_epi                = Repositories::SST_EPI;
        $this->sst_tipoepi            = Repositories::SST_TIPOEPI;
        $this->sst_perfilepi          = Repositories::SST_PERFILEPI;
        $this->sst_tipomedidaepi      = Repositories::SST_TIPOMEDIDAEPI;
        $this->sst_itemperfilepi      = Repositories::SST_ITEMPERFILEPI; 
        $this->sst_perfilepiocupacao  = Repositories::SST_PERFILEPIOCUPACAO;
        
        
    }
    
    public function validaDeleteEpi($nrorg,$nrepi) {
        $result = $this->epiRepository->gethistoricoepi($nrorg,$nrepi);
            if(!empty($result))
                return false;
        $result = $this->epiRepository->getitemperfilepi($nrorg,$nrepi);
            if(!empty($result))
                return false;
        $result = $this->epiRepository->getriscoepi($nrorg,$nrepi);
            if(!empty($result))
                return false;
        
        return true;
    }
    
    public function getFoto($row){
        return $this->epiRepository->getFoto($row);
    }
    
    public function baixarPlanilhaEpi() {
        $cabecalho = array(
            "Nome",
            "Certificado de Aprovação",
            "Qtd. Fornecida",
            "Antecipação (Dias)",
            "Proteção Eficaz (Sim ou Não)",
            "Devolução Obrigatória (Sim ou Não)",
            "Período (Meses)",
            "Tipo EPI",
            "Abreviação",
            "Tipo Medida EPI",
            "Observação",
            "Início Vigência",
            "Fim Vigência",
            "Ocupação"
        );

        $dadosExemplo[0] = [
            "Nome EPI",
            "CA_001",
            "1",
            "0",
            "Sim",
            "Não",
            "6",
            "Proteção",
            "",
            "Unidade",
            "",
            "01/01/2000",
            "",
            "ADVOGADO JUNIOR N1"
        ];
        
        return $this->exportacaoExcelService->gravarTemplate_1($this->hcmParameters->getNrorgTrabalho(), $cabecalho, 'EPI', $dadosExemplo);
    }
    
    public function importarPlanilhaEpi($row) {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        $dados       = $this->getDadosEpi($row['DSPATH'][0]);
        

        if(!empty($dados)) { 
            foreach ($dados as $lin => $dado) {
                $erros = $this->verificaErrosPlanilha($nrorg, $dado, $lin + 2);
                
                if (count($erros) > 0) {
                    foreach ($erros as $msg) {
                        $messages[] = $msg;
                    }
    
                } else {
                    $inserePlanilhaEpi    = $this->inserePlanilhaEpi($dado, $nrorg);
                    $inserePerfilEpi      = $this->getCreatePerfilEpi($dado, $nrorg);
                    $insereItemPerfil     = $this->insereItemPerfilEpi($dado, $nrorg, $inserePlanilhaEpi);
                    $inserePerfilOcupacao = $this->inserePerfilOcupacao($dado, $nrorg);
                }
            }
        }

        if(empty($dados)) {
            $messages[] =  array("msg" => "Não foi possível ler os dados da planilha. o campo 'Nome' não pode ser vazio", "row" => 2, "field" => "Nome");
            
        }
        
        return $messages;
    }
    
    
    public function verificaErrosPlanilha($nrorg, $dado, $linha) {
        $mensagens        = [];
        $nomeEpi          = $dado['NMEPI'];
        $nmtipoepi        = $dado['NMTIPOEPI'];
        $nmtipomedidaepi  = $dado['NMTIPOMEDIDAEPI'];
        $nmocupacaoh      = $dado['NMOCUPACAOH'];
        $qtforneceepi     = $dado['QTFORNECEEPI'];
        $qtdiasantecipa   = $dado['QTDIASANTECIPA'];
        $idprotecaoeficaz = $dado['IDPROTECAOEFICAZ'];
        $iddevolucaoobrig = $dado['IDDEVOLUCAOOBRIG'];
        $qtperiodmeses    = $dado['QTPERIODMESES'];
        $dsabreviacao     = $dado['DSABREVIACAO'];
        $nrctrlaprovepi   = $dado['NRCTRLAPROVEPI'];
        $dtinivigencia    = $dado['DTINIVIGENCIA'];
        $dtfimvigencia    = $dado['DTFIMVIGENCIA'];
        
        if($nomeEpi != null){
            if(strlen($nomeEpi) > 100 ){
                $mensagens[] = array("msg" => "Não é permitdo inserir mais de 100 caracteres no campo de nome.", "row" => $linha, "field" => 'Nome');
            }
            else{
                $nomeEpi     = mb_strtoupper($nomeEpi, "utf-8" );
                $objNomeEpi = $this->entityManager->getRepository(Repositories::SST_EPI)->findOneBy(array('nrorg' => $nrorg, 'nmepi' => $nomeEpi));
                if($objNomeEpi != null){
                    $mensagens[] = array("msg" => "Já existe EPI com o nome informado.", "row" => $linha, "field" => 'Nome');
                }
            }
        }
        else{
            $mensagens[] = array("msg" => "Não foi encontrado o nome para EPI, o campo não pode ser vazio.", "row" => $linha, "field" => 'Nome');
        }
        
        if($nmtipoepi == null){
            $mensagens[] = array("msg" => "Não foi encontrado tipo de EPI, o campo não pode ser vazio.", "row" => $linha, "field" => 'Tipo de Epi');
        }
        if(strlen($nmtipoepi) > 100 ){
            $mensagens[] = array("msg" => "Não é permitdo inserir mais de 100 caracteres no campo tipo de EPI.", "row" => $linha, "field" => 'Tipo de Epi');
        }
     
        if($nmtipomedidaepi == null){
            $mensagens[] = array("msg" => "Não foi encontrado tipo de medida EPI, o campo não pode ser vazio.", "row" => $linha, "field" => 'Tipo de Medida Epi');
        }
        if( strlen($nmtipomedidaepi) > 100 ){
            $mensagens[] = array("msg" => "Não é permitdo inserir mais de 100 caracteres no campo Tipo de Medida EPI.", "row" => $linha, "field" => 'Tipo de Medida Epi');
        }
        
        if($nmocupacaoh != null){
           $nmocupacaoh = mb_strtoupper($nmocupacaoh, "utf-8" );
           $objocupacaoh = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->findOneBy(array('nrorg' => $nrorg, 'nmocupacaoh' => $nmocupacaoh));
            if($objocupacaoh == null){
                $mensagens[] = array("msg" => "Não foi encontrada uma ocupação com o nome informado, por favor, verifique e tente novamente.", "row" => $linha, "field" => 'Ocupação');
            }
       }else{
            $mensagens[] = array("msg" => "Não foi encontrada a ocupação, o campo não pode ser vazio.", "row" => $linha, "field" => 'Ocupação');
       }
       
        if($qtforneceepi == null){
            $mensagens[] = array("msg" => "O campo de Quantidade Fornecida não pode ser vazio.", "row" => $linha, "field" => 'Quantidade Fornecida');
        }else{
            if(!is_numeric($qtforneceepi)){
            $mensagens[] = array("msg" => "O campo de quantidade fornecida deve ser númerico.", "row" => $linha, "field" => 'Quantidade Fornecida');
            }
        }
        
        if($qtdiasantecipa != null){
            if(!is_numeric($qtdiasantecipa)){
                $mensagens[] = array("msg" => "O campo de Antecipação  deve ser númerico.", "row" => $linha, "field" => 'Antecipação (Dias)');
            }
        }
        
        if($qtperiodmeses != null){
            if(!is_numeric($qtperiodmeses)){
                $mensagens[] = array("msg" => "O campo de Período (Meses)  deve ser númerico.", "row" => $linha, "field" => 'Período (Meses)');
            }
        }
        
        if($idprotecaoeficaz != null){
            if(preg_match('/\d+/', $idprotecaoeficaz)>0){
              $mensagens[] = array("msg" => "Não é permitdo inserir números no campo de Proteção Eficaz.", "row" => $linha, "field" => 'Proteção Eficaz');
            }
            $idprotecaoeficaz = mb_strtoupper($idprotecaoeficaz, "utf-8" );
            if($idprotecaoeficaz[0] != "S" && $idprotecaoeficaz[0] != "N"){
                $mensagens[] = array("msg" => "Não é permitdo inserir valor diferente de SIM ou NÃO no campo de Proteção Eficaz.", "row" => $linha , "field" => 'Proteção Eficaz');
            }
        }
        
        if($iddevolucaoobrig != null){
            if(preg_match('/\d+/', $iddevolucaoobrig)>0){
              $mensagens[] = array("msg" => "Não é permitdo inserir números no campo de Devolução Obrigatória.", "row" => $linha , "field" => 'Devolução Obrigatória');
            }
            $iddevolucaoobrig = mb_strtoupper($iddevolucaoobrig, "utf-8" );
            if($iddevolucaoobrig[0] != "S" && $iddevolucaoobrig[0] != "N"){
                $mensagens[] = array("msg" => "Não é permitdo inserir valor diferente de SIM ou NÃO no campo de Devolução Obrigatória.", "row" => $linha , "field" => 'Devolução Obrigatória');
            }
        }
        
        if($dsabreviacao != null){
            if(strlen($dsabreviacao)  > 14 ){
                $mensagens[] = array("msg" => "Não é permitdo inserir mais de 14 caracteres no campo de Abreviação.", "row" => $linha, "field" => 'Abreviação');
            }
            if(preg_match('/\d+/', $dsabreviacao)>0){
              $mensagens[] = array("msg" => "Não é permitdo inserir números no campo de Abreviação.", "row" => $linha, "field" => 'Abreviação');
            }
        }
        
        if( strlen($nrctrlaprovepi) > 30 ){
            $mensagens[] = array("msg" => "Não é permitdo inserir mais de 30 caracteres no campo de Certificado de Aprovação.", "row" => $linha, "field" => 'Certificado de Aprovação');
        }

        if(is_null($dtinivigencia)) {
            $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Início Vigência' com uma data válida.", "row" => $linha, "field" => "Início Vigência");
        }

        return $mensagens;
    }
    
    public function getCreateNrtipoepi($dado, $nrorg){
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $nmtipoepi = $dado['NMTIPOEPI'];
        $nmtipoepi = mb_strtoupper($nmtipoepi, "utf-8" );
        $objTipoEpi = $this->entityManager->getRepository(Repositories::SST_TIPOEPI)->findOneBy(array('nrorg' => array($nrorg, $nrorgPadrao), 'nmtipoepi' => $nmtipoepi));
        if($objTipoEpi == null){
            try{
                // new TIPO EPI
                $this->entityManager->beginTransaction();
                $tpepi = new $this->sst_tipoepi;
                $tpepi->setNrorg($nrorg);
                $tpepi->setNrtipoepi(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_TIPOEPI', 1, 12, true));
                $tpepi->setNmtipoepi($nmtipoepi);
                $tpepi->setDsabreviacao(null);
                $tpepi->setDsobservacao(null);
                
                // set auditoria TIPO EPI
                $tpepi->setDtinclusao(DateUtil::getDataAtual());
                $tpepi->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $tpepi->setNrorginclusao($nrorg);
                $tpepi->setDtultatu(DateUtil::getDataAtual());
                $tpepi->setCdoperultatu($this->hcmParameters->getCdoperador());
                $tpepi->setNrorgultatu($nrorg);
                $tpepi->setIdativo("S");
                
                $this->entityManager->persist($tpepi);
                $this->entityManager->flush($tpepi);
                $this->entityManager->commit();
                
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
            $nrTipoepi =  $tpepi->getNrtipoepi();
            return $nrTipoepi;
        }else{
            $nrTipoepi = $objTipoEpi->getNrtipoepi();
            return $nrTipoepi;
        }
        
    }  
    
    public function getCreateNrTipomedida($dado, $nrorg){
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $nmtipomedidaepi = $dado['NMTIPOMEDIDAEPI'];
        $nmtipomedidaepi = mb_strtoupper($nmtipomedidaepi, "utf-8" );
        $objTipoMedidaEpi = $this->entityManager->getRepository(Repositories::SST_TIPOMEDIDAEPI)->findOneBy(array('nrorg' => array($nrorg, $nrorgPadrao), 'nmtipomedidaepi' => $nmtipomedidaepi));
        if($objTipoMedidaEpi == null){
            try{
                // new TIPO MEDIDA EPI
                $this->entityManager->beginTransaction();
                $tpmedidaepi = new $this->sst_tipomedidaepi;
                $tpmedidaepi->setNrorg($nrorg);
                $tpmedidaepi->setNrtipomedidaepi(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_TIPOMEDIDAEPI', 1, 12, true));
                $tpmedidaepi->setNmtipomedidaepi($nmtipomedidaepi);
                $tpmedidaepi->setDsabreviacao(null);
                $tpmedidaepi->setDsobservacao(null);
                
                // set auditoria TIPO  MEDIDA EPI
                $tpmedidaepi->setDtinclusao(DateUtil::getDataAtual());
                $tpmedidaepi->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $tpmedidaepi->setNrorginclusao($nrorg);
                $tpmedidaepi->setDtultatu(DateUtil::getDataAtual());
                $tpmedidaepi->setCdoperultatu($this->hcmParameters->getCdoperador());
                $tpmedidaepi->setNrorgultatu($nrorg);
                $tpmedidaepi->setIdativo("S");
                
                $this->entityManager->persist($tpmedidaepi);
                $this->entityManager->flush($tpmedidaepi);
                $this->entityManager->commit();
                
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
            $nrTipoMedida = $tpmedidaepi->getNrtipomedidaepi();
            return $nrTipoMedida;
        }else{
            $nrTipoMedida = $objTipoMedidaEpi->getNrtipomedidaepi();
            return $nrTipoMedida;
        }
        
    }
    
    public function getCreatePerfilEpi($dado, $nrorg){
        $nmocupacaoh      = $dado['NMOCUPACAOH'];
        $nmocupacaoh      = mb_strtoupper($nmocupacaoh, "utf-8" );
        $dtinivigencia    = DateUtil::getDataDeString($dado['DTINIVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true);
        $dtfimvigencia    = isset($dado['DTFIMVIGENCIA']) ? DateUtil::getDataDeString($dado['DTFIMVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true) : null;
        $objPerfilOcupEpi = $this->entityManager->getRepository(Repositories::SST_PERFILEPI)->findOneBy(array('nrorg' => $nrorg, 'nmperfilepi' => $nmocupacaoh));
        if($objPerfilOcupEpi == null){
            try{
                // new PERFIL EPI
                $this->entityManager->beginTransaction();
                $perfilepi = new $this->sst_perfilepi;
                $perfilepi->setNrorg($nrorg);
                $perfilepi->setNrperfilepi(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_PERFILEPI', 1, 12, true));
                $perfilepi->setNmperfilepi($nmocupacaoh);
                $perfilepi->setDsobservacao(null);
                $perfilepi->setDtinivigencia($dtinivigencia);
                $perfilepi->setDtfimvigencia($dtfimvigencia);
                
                // set auditoria PERFIL EPI
                $perfilepi->setDtinclusao(DateUtil::getDataAtual());
                $perfilepi->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $perfilepi->setNrorginclusao($nrorg);
                $perfilepi->setDtultatu(DateUtil::getDataAtual());
                $perfilepi->setCdoperultatu($this->hcmParameters->getCdoperador());
                $perfilepi->setNrorgultatu($nrorg);
                $perfilepi->setIdativo("S");
                
                $this->entityManager->persist($perfilepi);
                $this->entityManager->flush($perfilepi);
                $this->entityManager->commit();
                
            }catch (\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
            $nrperfilepi  = $perfilepi->getNrperfilepi();
            return $nrperfilepi;
        }else{
            $nrperfilepi  = $objPerfilOcupEpi->getNrperfilepi();
            return $nrperfilepi;
        }
    }
    
    public function insereItemPerfilEpi($dado, $nrorg, $nrepi){
        $qtforneceepi     = $dado['QTFORNECEEPI'];
        $qtdiasantecipa   = $dado['QTDIASANTECIPA'];
        $idprotecaoeficaz = $dado['IDPROTECAOEFICAZ'];
        $iddevolucaoobrig = $dado['IDDEVOLUCAOOBRIG'];
        $qtperiodmeses    = $dado['QTPERIODMESES'];
        $dsabreviacao     = $dado['DSABREVIACAO'];
        $dsobservacao     = $dado['DSOBSERVACAO'];
        $idprotecaoeficaz = mb_strtoupper($idprotecaoeficaz, "utf-8" );
        $iddevolucaoobrig = mb_strtoupper($iddevolucaoobrig, "utf-8" );
        $dsabreviacao     = mb_strtoupper($dsabreviacao, "utf-8" );
        $dsobservacao     = mb_strtoupper($dsobservacao, "utf-8" );
        $dtinivigencia    = DateUtil::getDataDeString($dado['DTINIVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true);
        $dtfimvigencia    = isset($dado['DTFIMVIGENCIA']) ? DateUtil::getDataDeString($dado['DTFIMVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true) : null;
        $nrperfilepi      = $this->getCreatePerfilEpi($dado, $nrorg);
        try{
            // new PERFIL EPI
            $this->entityManager->beginTransaction();
            $perfilitemepi = new $this->sst_itemperfilepi;
            $perfilitemepi->setNrorg($nrorg);
            $perfilitemepi->setNrepi($nrepi);
            $perfilitemepi->setNrperfilepi($nrperfilepi);
            $perfilitemepi->setNritemperfilepi(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_ITEMPERFILEPI', 1, 12, true));
            $perfilitemepi->setQtforneceepi($qtforneceepi);
            $perfilitemepi->setQtdiasantecipa($qtdiasantecipa);
            $perfilitemepi->setIdprotecaoeficaz($idprotecaoeficaz[0]);
            $perfilitemepi->setIddevolucaoobrig($iddevolucaoobrig[0]);
            $perfilitemepi->setQtperiodmeses($qtperiodmeses);
            $perfilitemepi->setDtinivigencia($dtinivigencia);
            $perfilitemepi->setDtfimvigencia($dtfimvigencia);

            
            // set auditoria PERFIL EPI
            $perfilitemepi->setDtinclusao(DateUtil::getDataAtual());
            $perfilitemepi->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $perfilitemepi->setNrorginclusao($nrorg);
            $perfilitemepi->setDtultatu(DateUtil::getDataAtual());
            $perfilitemepi->setCdoperultatu($this->hcmParameters->getCdoperador());
            $perfilitemepi->setNrorgultatu($nrorg);
            $perfilitemepi->setIdativo("S");            
            
            $this->entityManager->persist($perfilitemepi);
            $this->entityManager->flush($perfilitemepi);
            $this->entityManager->commit();
            
        }catch (\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function inserePlanilhaEpi($dado, $nrorg){
        $nrTipoepi    = $this->getCreateNrtipoepi($dado, $nrorg);
        $nrTipoMedida = $this->getCreateNrTipomedida($dado, $nrorg);
        $nmepi        = $dado['NMEPI'];
        $dsabreviacao = $dado['DSABREVIACAO'];
        $dsobservacao = $dado['DSOBSERVACAO'];
        $nmepi        = mb_strtoupper($nmepi, "utf-8" );
        $nrctrlaprovepi   = $dado['NRCTRLAPROVEPI'];

        try{
            $this->entityManager->beginTransaction();
            // new EPI
            $epi = new $this->sst_epi;
            $epi->setNrorg($nrorg);
            $epi->setNmepi($nmepi);
            $epi->setNrtipomedidaepi($nrTipoMedida);
            $epi->setNrtipoepi($nrTipoepi);
            $epi->setDsabreviacao($dsabreviacao);
            $epi->setDsobservacao($dsobservacao);
            $epi->setNrctrlaprovepi($nrctrlaprovepi);
            $epi->setNrepi(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EPI', 1, 12, true));
            // set auditoria epi
            $epi->setDtinclusao(DateUtil::getDataAtual());
            $epi->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $epi->setNrorginclusao($nrorg);
            $epi->setDtultatu(DateUtil::getDataAtual());
            $epi->setCdoperultatu($this->hcmParameters->getCdoperador());
            $epi->setNrorgultatu($nrorg);
            $epi->setIdativo("S");

            $this->entityManager->persist($epi);
            $this->entityManager->flush($epi);
            $this->entityManager->commit();

            $nrEpi = $epi->getNrepi();
            return $nrEpi;
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function inserePerfilOcupacao($dado, $nrorg){
        $nrperfilepi   = $this->getCreatePerfilEpi($dado, $nrorg);
        $nmocupacaoh   = $dado['NMOCUPACAOH'];
        $nmocupacaoh   = mb_strtoupper($nmocupacaoh, "utf-8" );
        $objocupacaoh  = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->findOneBy(array('nrorg' => $nrorg, 'nmocupacaoh' => $nmocupacaoh));
        $nrocupacaom   = $objocupacaoh->getNrocupacaom();
        $objPerfilOcup = $this->entityManager->getRepository(Repositories::SST_PERFILEPIOCUPACAO)->findOneBy(array('nrorg' => $nrorg, 'nrperfilepi' => $nrperfilepi));
        $dtinivigencia = DateUtil::getDataDeString($dado['DTINIVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true);
        $dtfimvigencia = isset($dado['DTFIMVIGENCIA']) ? DateUtil::getDataDeString($dado['DTFIMVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true) : null;
        if($objPerfilOcup == null){
            try{
                $this->entityManager->beginTransaction();
                // new perfilepiocupacao
                $perfilepiocupacao = new $this->sst_perfilepiocupacao;
                $perfilepiocupacao->setNrorg($nrorg);
                $perfilepiocupacao->setNrperfilepiocupacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_PERFILEPIOCUPACAO', 1, 12, true));
                $perfilepiocupacao->setNrperfilepi($nrperfilepi);
                $perfilepiocupacao->setNrocupacaom($nrocupacaom);
                $perfilepiocupacao->setDsobservacao(null);
                $perfilepiocupacao->setDtinivigencia($dtinivigencia);
                $perfilepiocupacao->setDtfimvigencia($dtfimvigencia);
                
                // set auditoria perfilepiocupacao
                $perfilepiocupacao->setDtinclusao(DateUtil::getDataAtual());
                $perfilepiocupacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $perfilepiocupacao->setNrorginclusao($nrorg);
                $perfilepiocupacao->setDtultatu(DateUtil::getDataAtual());
                $perfilepiocupacao->setCdoperultatu($this->hcmParameters->getCdoperador());
                $perfilepiocupacao->setNrorgultatu($nrorg);
                $perfilepiocupacao->setIdativo("S");
    
                $this->entityManager->persist($perfilepiocupacao);
                $this->entityManager->flush($perfilepiocupacao);
                $this->entityManager->commit();
                
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
        }
        
    }
    
    public function getDadosEpi($path) {
        $head = array(
                "NMEPI",
                "NRCTRLAPROVEPI",
                "QTFORNECEEPI",
                "QTDIASANTECIPA",
                "IDPROTECAOEFICAZ",
                "IDDEVOLUCAOOBRIG",
                "QTPERIODMESES",
                "NMTIPOEPI",
                "DSABREVIACAO",
                "NMTIPOMEDIDAEPI",
                "DSOBSERVACAO",
                "DTINIVIGENCIA",
                "DTFIMVIGENCIA",
                "NMOCUPACAOH"
            );
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();


        $dateColumns = array('DTINIVIGENCIA','DTFIMVIGENCIA');
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NMEPI'), $dateColumns);
    }
    
}