<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\SstOcorrencia;
use Folha\Entities\SstVitima;
use Folha\Entities\SstFichamedica;
use Folha\Entities\SstVitimaparteatingida;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class OcorrenciaService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function deleteOcorrencia($dataset) {
        
        $nrorg = $dataset["NRORG"];
        $nrocorrencia = $dataset["NROCORRENCIA"];
        $vitima = SstVitima::findByNrocorrenciaAndNrorg($nrocorrencia, $nrorg);
        $fichamedica = SstFichamedica::findByNrocorrenciaAndNrorg($nrocorrencia, $nrorg);
        if (isset($vitima[0]) || isset($fichamedica[0])) {
            throw new \Exception(Messages::HCM05000_SST_OCORRENCIA_EXISTS_VITIMA_OR_FICHA_MEDICA_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $ocorrencia = SstOcorrencia::findOneByNrocorrenciaAndNrorg($nrocorrencia, $nrorg);
            $this->entityManager->remove($ocorrencia);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function deleteVitima($dataset) {
        $nrorg = $dataset["NRORG"];
        $nrvitima = $dataset["NRVITIMA"];
        $parteatingida = SstVitimaparteatingida::findByNrvitimaAndNrorg($nrvitima, $nrorg);
        if (isset($parteatingida[0])) {
            throw new \Exception(Messages::HCM05000_SST_VITIMA_EXISTS_PARTEATING_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $vitima = SstVitima::findOneByNrvitimaAndNrorg($nrvitima, $nrorg);
            $this->entityManager->remove($vitima);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function getNrvinculomByNrParcnegocio($nrorg, $nrparcnegocio, $dtmescompetenc) {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM
            FROM
                GPE_PESSOA PE
                INNER JOIN
                GPE_VINCULOM VM ON
                    VM.NRORG = PE.NRORG AND
                    VM.NRPESSOA = PE.NRPESSOA AND
                    (DTRESCISAOVINC >= :P_DTMESCOMPETENC OR DTRESCISAOVINC IS NULL)
            WHERE
                PE.NRORG = :P_NRORG AND
                PE.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $params = [
            'P_NRORG'           => $nrorg,
            'P_NRPARCNEGOCIO'   => $nrparcnegocio,
            'P_DTMESCOMPETENC'  => $dtmescompetenc
        ];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();
        return $result['NRVINCULOM'];
    }
}