<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\SstPerfilepi;
use Folha\Entities\SstItemperfilepi;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\SST\Repository\PerfilepiRepository;


class PerfilEpiService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, PerfilepiRepository $perfilepiRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->perfilepiRepository = $perfilepiRepository;
    }
    
    public function deletePerfilEpi($dataset) {
        
        $nrorg = $dataset["NRORG"];
        $nrperfilepi = $dataset["NRPERFILEPI"];
        
        $itemperfilepi = SstItemperfilepi::findByNrperfilepiAndNrorg($nrperfilepi, $nrorg);

        if (isset($itemperfilepi[0])) {
            throw new \Exception("Este perfil de EPI não pode ser deletado devido à existência de itens do perfil associados a ele.", 10);
        } 

        $perfilepiocupacao = $this->entityManager->getRepository(Repositories::SST_PERFILEPIOCUPACAO)->findOneBy(
            array('nrorg' => $nrorg, 'nrperfilepi' => $nrperfilepi));
           
        if (!is_null($perfilepiocupacao)) {
            throw new \Exception("Este perfil de EPI não pode ser deletado devido a uma associação com uma ocupação.", 10);
        }
  
        $perfilepiestrutura = $this->perfilepiRepository->buscaPerfilEpiAssocEstrutura($nrorg, $nrperfilepi);    
        if (isset($perfilepiestrutura[0])) {
            throw new \Exception("Este perfil de EPI não pode ser deletado devido a uma associação com uma estrutura.", 10);
        }

        $perfilepivinculo = $this->perfilepiRepository->buscaPerfilEpiAssocVinculo($nrorg, $nrperfilepi);
        if (isset($perfilepivinculo[0])) {
            throw new \Exception("Este perfil de EPI não pode ser deletado devido a uma associação com um vínculo.", 10);
        }
        
        $this->entityManager->beginTransaction();
        $perfilepi = SstPerfilepi::findOneByNrperfilepiAndNrorg($nrperfilepi, $nrorg);
        $this->entityManager->remove($perfilepi);
        $this->entityManager->flush();
        $this->entityManager->commit();

    }
}