<?php

namespace HCM\VAB\Controllers;

use Zeedhi\Framework\DTO\Response\Error;
use HCM\VAB\Services\BeneficioService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Folha\Entities\FpaFaixatabela;
use Folha\Entities\VabBeneficiario;
use Folha\Entities\VabBeneficioh;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Doctrine\ORM\EntityManager; 
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\AlertException;
use HCM\Geral\Services\CrudService;

class BeneficioController extends Crud {
 
	protected $crudService;
    protected $dataSourceName = 'getBeneficio';
    protected $dataSourceName2 = 'getBeneficioPesquisa';
    protected $parameterBagColumns = array('NRORG', 'NRORG_PADRAO','DTMESCOMPETENC');
    protected $beneficioService;
    
    const EXCLUSAO_AUTOMATICA = 'exclusao_automatica';
    const CADASTRO_AUTOMATICO = 'cadastro_automatico';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,BeneficioService $beneficioService,EntityManager $entityManager, CrudService $crudService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->beneficioService = $beneficioService;
        $this->entityManager = $entityManager;
		$this->crudService = $crudService;
        parent::__construct($dataSourceManager);
    }
    
  public function exportarTemplateBeneficio(DTO\Request\DataSet $request, DTO\Response $response){
    try{
        $file = $this->beneficioService->gravar_template();
        if($file){
            $return = array('NMARQUIVO' => $file);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
            
        }
        
    } catch(\Exception $e) {
        $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
    }
}
   
   public function importarPlanilhaBeneficio(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];

            $messages = $this->beneficioService->importarPlanilhaBeneficio($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
   
    
    public function getBeneficio (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 

        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            
            // Faixa Etária Titular
            if(isset($row['NRTABELAFOLHAPRI'])){
                $row['NRTABELAFOLHAPRI'] = explode(", ", $row['NRTABELAFOLHAPRI']);
                
                foreach($row['NRTABELAFOLHAPRI'] as $chave => $value)
                    $row['NRTABELAFOLHAPRI'][$chave] = intval($value);
                    
                $row['NMTABELAFOLHAPRI'] = explode(", ", $row['NMTABELAFOLHAPRI']);
            }    

            // Faixa Etária Dependente
            if(isset($row['NRTABELAFOLHASEC'])){
                $row['NRTABELAFOLHASEC'] = explode(", ", $row['NRTABELAFOLHASEC']);
                
                foreach($row['NRTABELAFOLHASEC'] as $chave => $value)
                    $row['NRTABELAFOLHASEC'][$chave] = intval($value);
                    
                $row['NMTABELAFOLHASEC'] = explode(", ", $row['NMTABELAFOLHASEC']);
            }    

            // Faixa Salarial Titular
            if(isset($row['NRTABFAIXASALTIT'])){
                $row['NRTABFAIXASALTIT'] = explode(", ", $row['NRTABFAIXASALTIT']);
                
                foreach($row['NRTABFAIXASALTIT'] as $chave => $value)
                    $row['NRTABFAIXASALTIT'][$chave] = intval($value);
                    
                $row['NMTABFAIXASALTIT'] = explode(", ", $row['NMTABFAIXASALTIT']);
            }    

            // Faixa Salarial Dependente
            if(isset($row['NRTABFAIXASALDEP'])){
                $row['NRTABFAIXASALDEP'] = explode(", ", $row['NRTABFAIXASALDEP']);
                
                foreach($row['NRTABFAIXASALDEP'] as $chave => $value)
                    $row['NRTABFAIXASALDEP'][$chave] = intval($value);
                    
                $row['NMTABFAIXASALDEP'] = explode(", ", $row['NMTABFAIXASALDEP']);
            }

        }

        $response->addDataSet(new DataSet($this->dataSourceName, $rows));    
    }

    public function getBeneficioPesquisa (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $filterData->addCondition('NRORG','=',$this->hcmParameters->getNrorgTrabalho());
        $filterData->addCondition('NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->addCondition('DTMESCOMPETENC','=',$this->hcmParameters->getDtCompetencia());
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet(new DataSet($this->dataSourceName, $dataSet->getRows()));    
    }

    public function getBeneficiarioPorEvento(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $nrbeneficiom = $conditions = null;
            $filters = $request->getFilterCriteria()->getConditions();
            foreach ($filters as $filter) {
                if($filter['columnName'] === 'NRBENEFICIOM'){
                    $nrbeneficiom = $filter['value'];
                }
            }
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $parameters = array($nrorg,$dtmescompetenc,$nrorgpadrao,$nrbeneficiom);
            Util::getValueInFilterDataSourceByName($filters, ['NRBENEFICIOM'], true);
            $conditions = Util::createQueryFiltered($filters);            

            $result = $this->beneficioService->getBeneficiarioPorEvento($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getBeneficiario', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getBeneficiario(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $nrbeneficiom = $conditions = null;
            $filters = $request->getFilterCriteria()->getConditions();
            foreach ($filters as $filter) {
                if($filter['columnName'] === 'NRBENEFICIOM'){
                    $nrbeneficiom = $filter['value'];
                }

                if($filter['columnName'] === 'NRBENEFICIOM'){
                    $nrbeneficiom = $filter['value'];
                }
            }
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $parameters = array($nrorg,$dtmescompetenc,$nrorgpadrao,$nrbeneficiom);
            
            Util::getValueInFilterDataSourceByName($filters, ['NRBENEFICIOM'], true);
            $conditions = Util::createQueryFiltered($filters);            
            $result = $this->beneficioService->getBeneficiario($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getBeneficiario', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaExistenciaTipoBeneficioGerencial(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $msg = null;
                $msg = $this->beneficioService->verificaTipoBeneficioGerencialExistente($row);
                
                if(!empty($msg)){
                     $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
                }
            }
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getTipoBeneficioGerencialVigente(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $dataSet = null;
                $dataSet = $this->beneficioService->getTipoBeneficioGerencialVigente($row);
                
                if(!empty($dataSet)){
                     $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTipoBeneficioGerencialVigente', $dataSet));
                }
            }
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadastroAutomaticoBeneficiario(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $action = $row['action'];
                $beneficiariosInvalidos = array();
                $nrvalorreferenciam = null;
                $nrvinculom_list = $row['NRVINCULOM_LIST'];
                $nrestrutsind  = isset($row['NRESTRUTSIND']) && !empty($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : 'A';
                $nrestrutlegal = isset($row['NRESTRUTLEGAL']) && !empty($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : 'A';
                $nrestrutgeren = isset($row['NRESTRUTGEREN']) && !empty($row['NRESTRUTGEREN']) ? $row['NRESTRUTGEREN'] : 'A'; 
                $nrpessoa_list = $row['NRPESSOA_LIST'];
                $nrbeneficiom = $row['NRBENEFICIOM'];
                $lista_nrpessoasvinc = $row['NRPESSOASVINC'];
                $vrbeneficio = isset($row['VRBENEFICIO']) ? $row['VRBENEFICIO'] : null;
                $qtbeneficio = $row['QTBENEFICIO'];
                $nrtabelafolhapri = $row['NRTABELAFOLHAPRI'];
                $nreventom_list = isset($row['NREVENTOM_LIST']) ? $row['NREVENTOM_LIST'] : null;
                $nrestruturam = $row['NRESTRUTURAM'];
                $tipovalor = isset($row['IDVALORDIFEBENEF']) ? $row['IDVALORDIFEBENEF'] : null;
                $nrtabelafolhasec = $row['NRTABELAFOLHASEC'];
                $tipobeneficiario = isset($row['IDTPBENEFICIARIO']) ? $row['IDTPBENEFICIARIO'] : null;
                $dtinscbeneficiario = isset($row['DTINSCBENEFICIARIO']) ? $row['DTINSCBENEFICIARIO'] : null;
                if(isset($row['NRVALORREFERENCIAM'])){
                    $nrvalorreferenciam = $row['NRVALORREFERENCIAM'] ? $row['NRVALORREFERENCIAM'] : null;
                }
                $tpperiodobenef = $row['TPPERIODOBENEF'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
                $lista_nrvinculom = is_array($nrvinculom_list) ? implode(',',array_values($nrvinculom_list)) : $nrvinculom_list;
                $lista_nrpessoas = is_array($nrpessoa_list) ? "_".implode('_',array_values($nrpessoa_list))."_" : $nrpessoa_list;
                switch($action) {
                    case self::EXCLUSAO_AUTOMATICA:
                        $this->beneficioService->removerBeneficiarios($lista_nrpessoas, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $nrbeneficiom, $nrorg, $lista_nrpessoasvinc, $dtmescompetenc, $nrorgpadrao, $nrvinculom_list, $tipobeneficiario, $nrestruturam, $nreventom_list);
                        break;
                    case self::CADASTRO_AUTOMATICO:
                        $beneficiariosInvalidos = $this->beneficioService->cadastroAutomatico($lista_nrpessoas, $nrestrutsind, $nrestrutlegal, $nrestrutgeren,$nrbeneficiom,$nrorg,$lista_nrpessoasvinc,$dtmescompetenc,$nrorgpadrao,$nrvinculom_list,$nrtabelafolhapri,$vrbeneficio,$qtbeneficio,$nreventom_list,$nrestruturam,$nrtabelafolhasec,$tpperiodobenef,$nrvalorreferenciam,$tipobeneficiario,$tipovalor,$dtinscbeneficiario);
                        if($beneficiariosInvalidos['CADASTRADOS'] !== '' || $beneficiariosInvalidos['SEMCPF'] !== ''){
                            /*--------------- Rotina para associação vínculo evento--
                            if($beneficiariosInvalidos['REFERENCIA'] === ''){
                               $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Valor de referencia informado para as pessoas abaixo está diferente do cadastrado para o evento '.$nreventom.':<br/> '.$beneficiariosInvalidos['REFERENCIA'], DTO\Response\Message::TYPE_WARNING));
                            }
                            else -------------------------------------------------------*/
                            if($beneficiariosInvalidos['CADASTRADOS'] === ''){
                                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A(s) pessoa(s) abaixo não possuem CPF: <br/>'.$beneficiariosInvalidos['SEMCPF'], DTO\Response\Message::TYPE_WARNING));
                            }
                            else if($beneficiariosInvalidos['SEMCPF'] === ''){
                                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A(s) pessoa(s) abaixo ja estava(m) cadastradas: <br/>'.$beneficiariosInvalidos['CADASTRADOS'], DTO\Response\Message::TYPE_WARNING));
                            }
                            else{
                                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('A(s) pessoa(s) abaixo ja estava(m) cadastradas: <br/>'.$beneficiariosInvalidos['CADASTRADOS'].'<br/> A(s) pessoa(s) abaixo não possuem CPF: <br/>'.$beneficiariosInvalidos['SEMCPF'], DTO\Response\Message::TYPE_WARNING));
                            }        
                        } 
                        break;
                }
            } 
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function setBeneficio(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === false){
                    $beneficiario = VabBeneficiario::findByNrbeneficiomAndNrorg($row['NRBENEFICIOM'],$row['NRORG']);
                    if (isset($beneficiario[0])) {
                        $beneficio = VabBeneficioh::findByNrbeneficiohAndNrorg($row['NRBENEFICIOH'],$row['NRORG']);

                        foreach ($beneficio as $beneficioRow) {
                            $estruturaOld = $beneficioRow->getNrestruturam();
                            $tpperiodobenefOld = $beneficioRow->getTpperiodobenef();
                        }

                        if($row['NRESTRUTURAM'] !== $estruturaOld){
                            //return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIO_ALT_ESTRUT_ERROR, DTO\Response\Message::TYPE_ERROR));
                        }

                        if($row['TPPERIODOBENEF'] !== $tpperiodobenefOld){
                            return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIO_ALT_TPPER_ERROR, DTO\Response\Message::TYPE_ERROR));
                        }
                    }
                }
                
                // Faixa Etária Titular
                if(isset($row['NRTABELAFOLHAPRI']) && is_array($row['NRTABELAFOLHAPRI']) )
                    $row['NRTABELAFOLHAPRI'] = implode(", ", $row['NRTABELAFOLHAPRI']);                        

                // Faixa Etária Dependente
                if(isset($row['NRTABELAFOLHASEC']) && is_array($row['NRTABELAFOLHASEC']) )
                    $row['NRTABELAFOLHASEC'] = implode(", ", $row['NRTABELAFOLHASEC']);                        

                // Faixa Salarial Titular
                if(isset($row['NRTABFAIXASALTIT']) && is_array($row['NRTABFAIXASALTIT']) )
                    $row['NRTABFAIXASALTIT'] = implode(", ", $row['NRTABFAIXASALTIT']);                        

                // Faixa Salarial Dependente
                if(isset($row['NRTABFAIXASALDEP']) && is_array($row['NRTABFAIXASALDEP']) )
                    $row['NRTABFAIXASALDEP'] = implode(", ", $row['NRTABFAIXASALDEP']);  
                
                $row['NRTABELAFOLHATERC'] = (isset($row['NRTABELAFOLHATERC']) ? $row['NRTABELAFOLHATERC'] : null);    
                $row['NRTABELAFOLHAFOUR'] = (isset($row['NRTABELAFOLHAFOUR']) ? $row['NRTABELAFOLHAFOUR'] : null);
            
                $row['NRESTRUTSIND'] = (isset($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : null);
                $row['NREVENTOCOPART'] = (isset($row['NREVENTOCOPART']) ? $row['NREVENTOCOPART'] : null);
                $row['NREVENTOCOPARTDEP'] = (isset($row['NREVENTOCOPARTDEP']) ? $row['NREVENTOCOPARTDEP'] : null);
                
            }
            $this->beneficioService->validaCamposVaziosBeneficio($row);
            $this->saveBeneficioCrud($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveBeneficioCrud($request, $response) {
		try {
			$this->dataSourceName = 'vab_beneficiom';
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$this->crudService->save($this->dataSourceName, $rows);
			
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
			
			$currentRow = current($request->getDataSet()->getRows());
            
			if(isset($currentRow['saveHistoryTable'])) {
                if (!$currentRow['__is_new']) {
                    if (!$this->checkHistory($response, $currentRow)) {
                        // is new history
                        $currentRow['__is_new'] = true;
                    } 
                }

				$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
				$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
					return $element[$tableMasterPK];
				});

				$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
				$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
				$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);

                // set history properties
                $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
                $this->throwExceptionIfRowIsNotUpdated($response, $currentRow);
                
	        	$newDataSet = new DataSet($this->dataSourceName, array($currentRow));
	        	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
	            $request->getMethod(), $request->getRoutePath(), $request->getUserId());
	            $rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
	            
				$this->crudService->save($this->dataSourceName, $rows);

			}
			$response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
		    if ($e instanceof AlertException) {
		        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
		        return;
		    }
			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
	}
	
	public function removerBeneficio(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "vab_beneficioh";

        $historicosBeneficio = null;
        foreach($request->getDataSet()->getRows() as $row) {
            $historicosBeneficio = $this->entityManager->getRepository(Repositories::VAB_BENEFICIOH)->findBy(["nrorg"=>intval($this->hcmParameters->getNrorgTrabalho()), "nrbeneficiom"=>$row['NRBENEFICIOM']]);
        }    

        if(count($historicosBeneficio)>1){
            parent::delete($request, $response);
            $this->entityManager->flush();
        }else
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Impossível excluir. Existe apenas um histórico para este benefício.", Messages::ALERT_MESSAGE_CODE));
    }
    
    public function checkHistory($response, $row) {
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_competenc'])]
            )
        );

        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
        $this->crudService->find($response, $newFilterCriteria);

        if (count($response->getDataSets()[0]->getRows()) > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public function throwExceptionIfRowIsNotUpdated($response, $row) {
        $dataSetIndex = count($response->getDataSets());
        
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_history_pk']),
                "operator" => "!=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_history_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_competenc'])]
            )
        );

        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
        $this->crudService->find($response, $newFilterCriteria);

        if (count($response->getDataSets()[$dataSetIndex]->getRows()) > 0) {
            throw new AlertException(Messages::HCM02014_VAB_COMPETENCIA_INVALIDA);
        }
    }
    
    public function setBeneficiarioPorEvento (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "vab_beneficiario_set";
            foreach($request->getDataSet()->getRows() as $row) {

                $pessoaBeneficiario = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($this->hcmParameters->getNrorgTrabalho(), $row['NRPESSOA'], $this->hcmParameters->getDtCompetencia());
                if(isset($row["__is_new"]) && $row["__is_new"] == true && !$pessoaBeneficiario->getNrcpfpessoa()){
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02014_VAB_BENEFICIARIO_SEM_CPF,Messages::ALERT_MESSAGE_CODE));
                }

                if(!array_key_exists('NRINTEGTRANSPM', $row )) {
                    $row['NRINTEGTRANSPM'] = null; 
                }

                $beneficiarios = VabBeneficiario::findByNrbeneficiomAndNrpessoaAndNrorgAndNrvinculom($row['NRBENEFICIOM'],$row['NRPESSOA'], $this->hcmParameters->getNrorgTrabalho(), $row['NRVINCULOM']);
                if($row['__is_new'] === true){
                    foreach ($beneficiarios as $beneficiario) {
                        if ($beneficiario->getNreventom() == $row['NREVENTOM']) {
                            return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIARIO_JA_CADASTRADO_ERROR,Messages::ERROR_MESSAGE_CODE));
                        }
                    }   
                }
                else{
                    foreach ($beneficiarios as $beneficiario) {
                        if (($beneficiario->getNrbeneficiario() != $row['NRBENEFICIARIO']) && ($beneficiario->getNreventom() == $row['NREVENTOM'])) 
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIARIO_JA_CADASTRADO_ERROR,Messages::ERROR_MESSAGE_CODE));
                    }  
                } 

                if(!array_key_exists('NRVALORREFERENCIAM', $row)){
                    $row['NRVALORREFERENCIAM'] = null;
                }
                if(array_key_exists('NRBENEFICIARIO', $row)){
                    if($row['NRBENEFICIARIO'] === '' ){
                        $row['NRBENEFICIARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
                    }
                } else {
                    $row['NRBENEFICIARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
                }


                if($row['IDVALORDIFEBENEF'] === 'TABELA'){
                    if($row['IDTPBENEFICIARIO'] === 'T'){                       
                        if($row["NRTABELAFOLHAPRI"]){
                            $tabelafolhapri = FpaFaixatabela::findByNrtabelafolha($row["NRTABELAFOLHAPRI"]);
                            $tabela = $tabelafolhapri;
                        }
                    }else{    
                        if($row["NRTABELAFOLHASEC"]){
                            $tabelafolhasec = FpaFaixatabela::findByNrtabelafolha($row["NRTABELAFOLHASEC"]);
                            $tabela = $tabelafolhasec;
                        }
                    }
                    if(empty($row['DTNASCIMENTO'])){
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::evaluateMessage(Messages::HCM00000_GER_DTNASC_PESS_N_INF_ERROR, ''), Messages::ERROR_MESSAGE_CODE));
                    }
                    $vrbeneficio = $this->beneficioService->calculavrtabela($row['DTNASCIMENTO'],$tabela);
                    $row['VRBENEFICIO'] = $vrbeneficio;
                    if(empty($vrbeneficio)){
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIO_FX_ETARIA_N_ENCONT_ERROR,Messages::ERROR_MESSAGE_CODE));
                    }
                }
                if($row['IDVALORDIFEBENEF'] === 'REFERENCIA'){
                    if($row['__is_new'] === true){
                        /*--------------- Rotina para associação vínculo evento--
                        if($row['TPPERIODOBENEF'] !== 'DIARIO'){
                            $row['NRVINCEVENTO'] = $this->beneficioService->salvarSomarVrRefFixoVincEvento($row['NREVENTOM'],$row['NRESTRUTURAM'],$row['NRVINCULOM'],$row);
                        }else{
                            $row['NRVINCEVENTO'] = null;
                        }-------------------------------------------------------*/
                    }else{
                        $qtbeneficio = $row['QTBENEFICIO'];
                        
                        /*--------------- Rotina para associação vínculo evento--
                        if($row['TPPERIODOBENEF'] !== 'DIARIO'){
                            foreach ($beneficiario as $rowbeneficiario) {
                                $qtbeneficioOld = $rowbeneficiario->getQtbeneficio();
                            }
                            if($row['QTBENEFICIO'] != $qtbeneficioOld){
                                $row['QTBENEFICIO'] -= $qtbeneficioOld;
                            }else{
                                $row['QTBENEFICIO'] = 0;
                            }
                            $row['NRVINCEVENTO'] = $this->beneficioService->salvarSomarVrRefFixoVincEvento($row['NREVENTOM'],$row['NRESTRUTURAM'],$row['NRVINCULOM'],$row);
                        }else{
                            $row['NRVINCEVENTO'] = null;
                        }-------------------------------------------------------*/

                    }
                    
                    //----- Rotina adicionada para adaptação--
                    $row['NRVINCEVENTO'] = null;
                    $row['VRBENEFICIO'] = null;
                    //----------------------------------------
                        
                    if($row['NRVINCEVENTO'] === '!REFERENCIA'){
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("O evento relacionado cadastrado possui tipo de valor diferente do informado".$row['NREVENTOM'],Messages::ERROR_MESSAGE_CODE));
                    }
                }else{

                    if($row['__is_new'] === true){
                        /*--------------- Rotina para associação vínculo evento--
                        if($row['TPPERIODOBENEF'] !== 'DIARIO'){
                            $row['NRVINCEVENTO'] = $this->beneficioService->salvarSomarVrEventoFixoVincEvento($row['NREVENTOM'],$row['NRESTRUTURAM'],$row['NRVINCULOM'], $row['VRBENEFICIO'], null, $row);
                        }else{
                            $row['NRVINCEVENTO'] = null;
                        }-------------------------------------------------------*/    
                    }else{
                        if($row['TPPERIODOBENEF'] !== 'DIARIO'){
                            
                            /*--------------- Rotina para associação vínculo evento-                            
                            foreach ($beneficiario as $rowbeneficiario) {
                                $vrbeneficioOld = $rowbeneficiario->getVrbeneficio();
                            }-------------------------------------------------------*/    
                            
                            $vrbeneficioNew = $row['VRBENEFICIO'] * $row['QTBENEFICIO'];
                            $row['VRBENEFICIO'] = $vrbeneficioNew;
                            
                            /*--------------- Rotina para associação vínculo evento-                            
                            if($vrbeneficioNew !== $vrbeneficioOld){
                                $vrbeneficioNew -= $vrbeneficioOld;
                                $editarbeneficiario = 'S';
                                $row['NRVINCEVENTO'] = $this->beneficioService->salvarSomarVrEventoFixoVincEvento($row['NREVENTOM'], $row['NRESTRUTURAM'], $row['NRVINCULOM'], $vrbeneficioNew, $vrbeneficioOld, $row, $editarbeneficiario);
                                $row['NRVINCEVENTO'] = null;
                            }-------------------------------------------------------*/    

                        }
                        /*--------------- Rotina para associação vínculo evento- 
                        else{
                            $row['NRVINCEVENTO'] = null;
                        }-------------------------------------------------------*/    
                    }
                    //----- Rotina adicionada para adaptação--
                    $row['NRVINCEVENTO'] = null;
                    //----------------------------------------
                }
                
                $this->beneficioService->validaCamposVaziosBeneficiarioPorEvento($row);
            }
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("success"));
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setBeneficiario (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "vab_beneficiario_set";
            foreach($request->getDataSet()->getRows() as $row) {
                $pessoaBeneficiario = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($this->hcmParameters->getNrorgTrabalho(), $row['NRPESSOA'], $this->hcmParameters->getDtCompetencia());
                
                
                if(isset($row["__is_new"]) && $row["__is_new"] == true && !$pessoaBeneficiario->getNrcpfpessoa()){
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02014_VAB_BENEFICIARIO_SEM_CPF,Messages::ALERT_MESSAGE_CODE));
                }
                
                $beneficiario = VabBeneficiario::findByNrbeneficiomAndNrpessoaAndNrorgAndNrvinculom($row['NRBENEFICIOM'],$row['NRPESSOA'], $this->hcmParameters->getNrorgTrabalho(), $row['NRVINCULOM']);
                
                if($row['__is_new'] === true){
                    if (isset($beneficiario[0])) {
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIARIO_JA_CADASTRADO_ERROR,Messages::ERROR_MESSAGE_CODE));
                    }
                }
                else if ( isset($beneficiario[0]) && ($beneficiario[0]->getNrbeneficiario() != $row['NRBENEFICIARIO']) ) 
                    return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM02000_VAB_BENEFICIARIO_JA_CADASTRADO_ERROR,Messages::ERROR_MESSAGE_CODE));
    
                if(array_key_exists('NRBENEFICIARIO', $row)){
                    if($row['NRBENEFICIARIO'] === '' ){
                        $row['NRBENEFICIARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
                    }
                } else {
                    $row['NRBENEFICIARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
                }
                
                if($row['IDTPBENEFICIARIO'] == 'T' && $row['IDATIVO'] == 'S') {
                    $temEventoDuplicado = $this->beneficioService->validaBeneficios($row);

                    if($temEventoDuplicado[0]) {
                        $message = Messages::evaluateMessage(Messages::HCM02000_VAB_BENEFICIARIO_EVENTO_DUPLICADO_ERROR, $temEventoDuplicado[1]);

                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message, Messages::ERROR_MESSAGE_CODE));
                    }
                }

                if($row['IDTPBENEFICIARIO'] == 'T' && $row['DTCANCBENEFICIARIO']){
                    $this->beneficioService->insereDtCancelamentoDependentes($row['NRVINCULOM'], $row['NRBENEFICIOM'], $row['DTCANCBENEFICIARIO']);
                }
                
                $this->beneficioService->validaCamposVaziosBeneficiario($row);
            }
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("success"));
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removerBeneficiario(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "vab_beneficiario_set";
        $dataSet = $request->getDataSet()->getRows();
        parent::delete($request, $response);
        $this->entityManager->flush();
        /*--------------- Rotina para associação vínculo evento--
        foreach ($dataSet as $row) {
            if($row['IDVALORDIFEBENEF'] === 'REFERENCIA'){
                $this->beneficioService->subtrairVrRefFixoVincEvento($row['NREVENTOM'],$row['NRESTRUTURAM'],$row['NRVINCULOM'],$row['QTBENEFICIO']);
            }else{
                $this->beneficioService->subtrairVrEventoFixoVincEvento($row['NREVENTOM'],$row['NRESTRUTURAM'],$row['NRVINCULOM'],$row['VRBENEFICIO'],$row['QTBENEFICIO'], $row['IDVALORDIFEBENEF']);
            }
        }-------------------------------------------------------*/
    }
    
    public function getBeneficioPorVinculo(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $filters = array();
            foreach($conditions as $condition){
                if($condition['value'] === 'NULL'){
                    $condition['value'] = NULL;
                }
                $filters[$condition['columnName']] = $condition['value'];
            }
            if(!array_key_exists('P_IDTPBENEFICIARIO', $filters)){
                $filters['P_IDTPBENEFICIARIO'] = null;
            }
            if(!array_key_exists('P_NRPESSOA_DEP', $filters)){
                $filters['P_NRPESSOA_DEP'] = null;
            }
            Util::getValueInFilterDataSourceByName($conditions, array('NRORG', 'DTMESCOMPETENC', 'P_IDTPBENEFICIARIO', 'P_NRVINCULOM', 'P_NRPESSOA_DEP', 'IDTPBENEFICIARIO', 'P_NRORG_PADRAO'), true);
            $filtro =  Util::createQueryFiltered($conditions, 'A');
            
            $beneficios  = $this->beneficioService->getBeneficioPorVinculo($filters['DTMESCOMPETENC'], $filters['NRORG'], $filters['P_IDTPBENEFICIARIO'], $filters['P_NRVINCULOM'], $filters['P_NRPESSOA_DEP'], $filters['P_NRORG_PADRAO'], $filtro);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getBeneficioPorVinculo', $beneficios));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getBeneficiarioVinculo(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $filters = array();
            foreach($conditions as $condition){
                if($condition['value'] === 'NULL'){
                    $condition['value'] = NULL;
                }
                $filters[$condition['columnName']] = $condition['value'];
            }
            
            $filters['IDTPBENEFICIARIO'] = array_key_exists('IDTPBENEFICIARIO', $filters) ? $filters['IDTPBENEFICIARIO'] : null;
            $filters['IDATIVO'] = array_key_exists('IDATIVO', $filters) ? $filters['IDATIVO'] : null;
            Util::getValueInFilterDataSourceByName($conditions, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRVINCULOM', 'P_NRORG_PADRAO', 'IDTPBENEFICIARIO', 'IDATIVO'), true);
            $filtro =  Util::createQueryFiltered($conditions, 'A');
            
            $beneficios  = $this->beneficioService->retornaBefeficiarioVinculo($filters['P_NRORG'], $filters['P_NRORG_PADRAO'], $filters['P_NRVINCULOM'], $filters['P_DTMESCOMPETENC'], $filters['IDTPBENEFICIARIO'], $filters['IDATIVO'], $filtro);
           
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getBeneficiarioVinculo', $beneficios));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
  
    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
            $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}