<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 01/09/2016
 * Time: 16:25
 */

namespace HCM\VAB\Controllers;

use Folha\Entities\FpaTipomoviment;
use HCM\Util\Messages;
use HCM\Util\FPA;
use HCM\VAB\Services\LancamentoBeneficioService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class LancamentoBeneficioController extends Crud {

    protected $dataSourceName = 'vab_histbenefvinc';
    protected $dataSourceName_get = 'getHistbenefvinc';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO', 'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER');

    const GRAVAR_TOTAL = 'gravar_total';
    const GRAVAR_PARCIAL = 'gravar_parcial';
    const RETROCEDER_ITENS = 'retroceder_itens';
    const EXPORTAR_LOTE = 'exportar_lote';
    const EXPORTAR_LOTE_LISTAS = 'exportar_lote_listas';
    const CADASTRO_DIAS = 'cadastro_dias';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, LancamentoBeneficioService $lancamentoBeneficioService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->lancamentoBeneficioService = $lancamentoBeneficioService;
        parent::__construct($dataSourceManager);
    }

    public function getHistbenefvinc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        
        $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
        $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());
        
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setHistbenefvinc (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {

                if (array_key_exists('DTMESCOMPETENC', $row)) {
                    if ($row['DTMESCOMPETENC'] === '') {
                        $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
                    }
                } else {
                    $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
                }

                $nrcalculomensal = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_MENSAL);
                $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);

                $this->lancamentoBeneficioService->verificaEventoVinculoExportadoFolha($nrcalculomensal, $row['NRVINCULOM'], $row['NREVENTOM'], $row['NRORG']);
                $this->lancamentoBeneficioService->verificaVinculoLiquidoGravado($nrcalculobenef, $row['NRVINCULOM'], $row['NRORG']);

                if (array_key_exists('NRCALCULOFOLHA', $row)) {
                    if ($row['NRCALCULOFOLHA'] === '') {
                        $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                    }
                } else {
                    $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                }

                if (array_key_exists('NRHISTBENEFVINC', $row)) {
                    if ($row['NRHISTBENEFVINC'] === '') {
                        $row['NRHISTBENEFVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_HISTBENEFVINC', 1, 12, true);
                    }
                } else {
                    $row['NRHISTBENEFVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_HISTBENEFVINC', 1, 12, true);
                }
                
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrcalculomensal = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_MENSAL);
                $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
                $this->lancamentoBeneficioService->verificaExportardosFolha($nrorg, $dtmescompetenc, $nrcalculomensal, $row['NRBENEFICIOM'], null, null, null, '_'.$row['NRVINCULOM'].'_');
                $this->lancamentoBeneficioService->verificaLiquidoGravado($nrorg, $dtmescompetenc, $nrcalculobenef, $row['NRBENEFICIOM'], null, null, null, '_'.$row['NRVINCULOM'].'_',null, null, null, null);
                $this->lancamentoBeneficioService->excluiItensBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal, $row['NRBENEFICIOM'], null, null, null, '_'.$row['NRVINCULOM'].'_', null, null, null, null);
            }
            parent::delete($request, $response);
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function processaLancamentoBeneficio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                
                $action = $row['action'];

                $nrorg          = $this->hcmParameters->getNrorgTrabalho();
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();

                $nrcalculomensal = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_MENSAL);
                $nrcalculobenef  = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);

                $nrbeneficiom          = isset($row['NRBENEFICIOM'])          ? $row['NRBENEFICIOM']          : null;
                $nrtpbeneficio         = isset($row['NRTPBENEFICIO'])         ? $row['NRTPBENEFICIO']         : null;
                $nrestrutlegal         = isset($row['NRESTRUTLEGAL'])         ? $row['NRESTRUTLEGAL']         : null;
                $nrestrutgeren         = isset($row['NRESTRUTGEREN'])         ? $row['NRESTRUTGEREN']         : null;
                $nrestrutura_beneficio = isset($row['NRESTRUTURA_BENEFICIO']) ? $row['NRESTRUTURA_BENEFICIO'] : null;
                $dtiniadmissao         = isset($row['DTINIADMISSAO'])         ? $row['DTINIADMISSAO']         : null;
                $dtfimadmissao         = isset($row['DTFIMADMISSAO'])         ? $row['DTFIMADMISSAO']         : null;
                $dsnovolote            = isset($row['DSNOVOLOTE'])            ? $row['DSNOVOLOTE']            :null;
                $obsnovolote           = isset($row['OBSNOVOLOTE'])           ? $row['OBSNOVOLOTE']           :null;
    
                if (isset($row['NRVINCULOM_LIST'])) {
                    $lista_nrvinculom = is_array($row['NRVINCULOM_LIST']) ? "_" . implode('_', array_values($row['NRVINCULOM_LIST'])) . "_" : $row['NRVINCULOM_LIST'];
                }
                if (isset($row['NRESTRUTLEGAL_LIST'])) {
                    $lista_nrestrutlegal = is_array($row['NRESTRUTLEGAL_LIST']) ? "_" . implode('_', array_values($row['NRESTRUTLEGAL_LIST'])) . "_" : $row['NRESTRUTLEGAL_LIST'];
                }
                if (isset($row['NRESTRUTGEREN_LIST'])) {
                    $lista_nrestrutgeren = is_array($row['NRESTRUTGEREN_LIST']) ? "_" . implode('_', array_values($row['NRESTRUTGEREN_LIST'])) . "_" : $row['NRESTRUTGEREN_LIST'];
                }
                if (isset($row['NRESTRUTSIND_LIST'])) {
                    $lista_nrestrutsind = is_array($row['NRESTRUTSIND_LIST']) ? "_" . implode('_', array_values($row['NRESTRUTSIND_LIST'])) . "_" : $row['NRESTRUTSIND_LIST'];
                }
                if (isset($row['NRESTRUTBENEF_LIST'])) {
                    $lista_nrestrutbenef = is_array($row['NRESTRUTBENEF_LIST']) ? "_" . implode('_', array_values($row['NRESTRUTBENEF_LIST'])) . "_" : $row['NRESTRUTBENEF_LIST'];
                }
                if (isset($row['NRTPBENEFICIO_LIST'])) {
                    $lista_nrtpbeneficio = is_array($row['NRTPBENEFICIO_LIST']) ? "_" . implode('_', array_values($row['NRTPBENEFICIO_LIST'])) . "_" : $row['NRTPBENEFICIO_LIST'];
                }
                if (isset($row['NRBENEFICIOM_LIST'])) {
                    $lista_nrbeneficiom = is_array($row['NRBENEFICIOM_LIST']) ? "_" . implode('_', array_values($row['NRBENEFICIOM_LIST'])) . "_" : $row['NRBENEFICIOM_LIST'];
                }
                if (isset($row['NREVENTOM_LIST'])) {
                    $lista_nreventom = is_array($row['NREVENTOM_LIST']) ? "_" . implode('_', array_values($row['NREVENTOM_LIST'])) . "_" : $row['NREVENTOM_LIST'];
                }
                if (isset($row['NRLOTEVINC_LIST'])) {
                    $lista_lotevinculos = is_array($row['NRLOTEVINC_LIST']) ? "_" . implode('_', array_values($row['NRLOTEVINC_LIST'])) . "_" : $row['NRLOTEVINC_LIST'];
                } else {
                    $lista_lotevinculos = null;
                }

                switch($action) {
                    case self::GRAVAR_TOTAL:
                        $this->lancamentoBeneficioService->gravarTotal($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                            $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
                        break;
                    case self::GRAVAR_PARCIAL:
                        $this->lancamentoBeneficioService->gravarParcial($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                            $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
                        break;
                    case self::RETROCEDER_ITENS:
                        $this->lancamentoBeneficioService->retrocederItens($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                            $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal);
                        break;
                    case self::EXPORTAR_LOTE:
                        if (isset($row['ITENS_LANCAMENTO_BENEFICIO']) && $row['ITENS_LANCAMENTO_BENEFICIO']) {
                            $this->lancamentoBeneficioService->exportarLoteItensBeneficio(
                                $nrorg, $dtmescompetenc, $nrcalculobenef, $nrestrutlegal, $nrestrutgeren, $nrestrutura_beneficio, $lista_nrvinculom, $row
                            );
                        } else {
                            $this->lancamentoBeneficioService->exportarLote(
                                $nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, 
                                $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $lista_nrestrutlegal
                            );
                        }
                        break;
                    case self::EXPORTAR_LOTE_LISTAS:
                        $this->lancamentoBeneficioService->exportarLoteItensBeneficioListas(
                            $nrorg, $dtmescompetenc, $nrcalculobenef, $lista_nrestrutlegal, $lista_nrestrutgeren, $lista_nrestrutsind, $lista_nrvinculom,
                            $lista_nrestrutbenef, $lista_nrbeneficiom, $lista_nrtpbeneficio, $lista_nreventom, $row
                        );
                        break;
                    case self::CADASTRO_DIAS:
                        $this->lancamentoBeneficioService->cadastroDias($nrorg, $dtmescompetenc, $nrcalculobenef, $nrcalculomensal,
                            $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $row['QTDIASACRE'], $row['QTDIASDESC']);
                        break;
                        
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function importaUnimed(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $message = array();
                
                foreach ($row['DSPATH'] as $arquivo) {
                    $row['DSPATH'] = [$arquivo];
                    $message = array_merge($message, $this->lancamentoBeneficioService->importaUnimed($row));
                }
                
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMessage', [$message]));
                $response->addNotification(new DTO\Response\Notification(Messages::HCM00001_GER_IMPORTACAO_SUCESS));
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}