-- ============================================================
--   Database name:  FOLHA                                   
--   DBMS name:      ORACLE Version 12.x 
--   Created on:     29/07/2021  12:01                         
-- ============================================================

-- ============================================================
--   Table: PTO_PASSOSUSOFREQ                                      
-- ============================================================
/* TABELA COM PASSOS DE ORG. 0/ORG DO CLIENTE */
CREATE TABLE PTO_PASSOSUSOFREQ
(
    NRORG                   NUMBER                              NOT NULL,
    NRPASSOSUSOFREQ         NUMBER                              NOT NULL,
    DSPASSOSUSOFREQ         VARCHAR2(4000)                      NOT NULL,
    OBSERVACAO              VARCHAR2(4000)                      NULL,
    DSREDIRECT              VARCHAR2(4000)                      NULL,       /* NOME DO CONTAINER NO FRAMEWORK ATUAL */
    DSREDIRECTSECTION       VARCHAR2(4000)                      NULL,       /* NOMA DA WIDGET NO FRAMEWORK ATUAL */
    NRORDEMUSOFREQ          NUMBER                              NOT NULL,
    IDATIVO                 VARCHAR2(1 BYTE)        DEFAULT 'S', 
    NRORGINCLUSAO           NUMBER                  DEFAULT 1   NULL, 
    DTINCLUSAO              DATE                                NOT NULL,
    CDOPERINCLUSAO          VARCHAR2(12 BYTE)       DEFAULT '1' NULL,
    NRORGULTATU             NUMBER                  DEFAULT 1   NULL, 
    DTULTATU                DATE                                NULL, 
    CDOPERULTATU            VARCHAR2(12 BYTE)       DEFAULT '1' NULL, 	
    CONSTRAINT PK_PTO_PASSOSUSOFREQ PRIMARY KEY (NRPASSOSUSOFREQ),
    CONSTRAINT CKC_IDATIVO_PTO_PASSOSUSOFREQ CHECK (IDATIVO IN ('S','N')) ENABLE
)
/

-- ============================================================
--   Table: PTO_PERIODOUSOFREQ                                      
-- ============================================================
CREATE TABLE PTO_PERIODOUSOFREQ
(
    NRORG                   NUMBER                                  NOT NULL,
    NRPERIODOUSOFREQ        NUMBER                                  NOT NULL,
    NRPASSOSUSOFREQ         NUMBER                                  NOT NULL,
    NRPERIODOAPURACAO       NUMBER                                  NOT NULL,
    DTULTOPER               DATE                    DEFAULT SYSDATE NOT NULL,
    IDATIVO                 VARCHAR2(1 BYTE)        DEFAULT 'S', 
    NRORGINCLUSAO           NUMBER                  DEFAULT 1       NULL, 
    DTINCLUSAO              DATE                    NOT NULL,
    CDOPERINCLUSAO          VARCHAR2(12 BYTE)       DEFAULT '1'     NULL,
    NRORGULTATU             NUMBER                  DEFAULT 1       NULL, 
    DTULTATU                DATE                                    NULL, 
    CDOPERULTATU            VARCHAR2(12 BYTE)       DEFAULT '1'     NULL, 	
    CONSTRAINT PK_PTO_PERIODOUSOFREQ PRIMARY KEY (NRORG, NRPERIODOUSOFREQ),
    CONSTRAINT PERIODOUSOFREQ_UN1 UNIQUE (NRORG, NRPERIODOAPURACAO, NRPASSOSUSOFREQ) ENABLE,
    CONSTRAINT CKC_IDATIVO_PTO_PERIODOUSOFREQ CHECK (IDATIVO IN ('S','N')) ENABLE,
    CONSTRAINT FK_NRPASSOSUSOFREQ FOREIGN KEY (NRPASSOSUSOFREQ)
	  REFERENCES PTO_PASSOSUSOFREQ (NRPASSOSUSOFREQ) ENABLE,
    CONSTRAINT FK_NRPERIODOAPURACAO FOREIGN KEY (NRORG, NRPERIODOAPURACAO)
	  REFERENCES PTO_PERIODOAPURACAO (NRORG, NRPERIODOAPURACAO) ENABLE
)
/

/************************************************************************************************************************************************************
*                                                                                                                                                           *
* INSERTS DE INFORMACOES AS TABELAS CRIADAS NA MODELAGEM DA TAREFA '342480 - [HCM-PASSOS DO ROTEIRO] - Roteiro de Uso - Módulo de Controle de Frequência'   *
* DATA: 30/07/2021                                                                                                                                          *
*                                                                                                                                                           *
*************************************************************************************************************************************************************/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    1,
    'Importação de Marcações',
    'Possibilita a inclusão manual das marcações de ponto do período e/ou importação do arquivo .txt conforme layout Portaria MTE 1.510/2009 do REP (Registrador Eletrônico de Ponto).',
    'hcm07003_pto_marcacaoPonto',
    'MarcPonto',
    10,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    2,
    'Ausências Justificadas (Afastamento, Férias, Atestado Médico)',
    'Possibilita a verificação de ocorrências do tipo "Abono" dentre as marcações do período para osrespectivos vínculos e, através da página de ocorrências da tela de marcação, permitirá proceder com a manutenção dos dias do respectivo período de apuração e, eventualmente, incluir uma ou mais justificativas de abonos para os dias em que forem encontradas ausências (parcial ou total) demarcação.',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    20,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    3,
    'Ausências Injustificadas (Faltas e Atrasos)',
    'Possibilita a verificação de ocorrências do tipo "Falta" e/ou "Atraso" dentre as marcações do período para os respectivos vínculos e, através da página de ocorrências da tela de marcação, permitirá proceder com a manutenção dos dias do respectivo período de apuração e, eventualmente, incluir uma ou mais justificativas de abonos para os dias em que forem encontradas ausências (parcial ou total) de marcação',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    30,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    4,
    'Verificação de marcações inválidas (ímpar)',
    'Possibilita a verificação de ocorrências do tipo "Ímpar" dentre as marcações do período para os respectivos vínculos e, através da própria tela de marcação, o operador poderá proceder com ainclusão de marcação faltante ou exclusão de marcação excedente.',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    40,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    5,
    'Verificação de marcações intrajornada',
    'Possibilita a verificação de ocorrências do tipo "Intrajornada" que se referem às marcações cujo intervalo de horários ficou aquém do intervalo definido pelo horário base.',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    50,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    6,
    'Verificação de marcações interjornada',
    'Possibilita a verificação de ocorrências do tipo "Interjornada" que se referem às marcações cujo intervalo entre as jornadas ficou aquém do previsto pela legislação (Artigo 66 CLT).',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    60,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    7,
    'Verificação de horas extras',
    'Possibilita a verificação de ocorrências do tipo "Extra" que se referem às marcações cujo dia gerou um excedente de horas previamente definido pela escala de trabalho do vínculo.',
    'hcm07003_pto_marcacaoPonto',
    'ocorrencias',
    70,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    8,
    'Apuração do Ponto',
    'Possibilita a abertura da tela contendo o formulário para a realização Cálculo da Apuração de Frequência do respectivo período.',
    'hcm07022_pto_apuracaoFrequencia',
    NULL,
    80,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    9,
    'Resumos Gerais da Apuração',
    'Possibilita a abertura da tela contendo o formulário para a realização Resumos Gerais da Apuração do  respectivo período para possibilitar ao operador verificar o resultado consolidado da apuração defrequência do período.',
    'hcm07006_pto_resumosGeraisApuracao',
    'resumosGeraisApuracaoPeriodo',
    90,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    10,
    'Geração de Lote de Eventos de Folha de Pagamento',
    'Possibilita a abertura da tela contendo o formulário para a realização da geração do lote de eventos oriundos do resumo de frequência da apuração do respectivo período.',
    'hcm07019_pto_geracaoLoteEventosFolha',
    NULL,
    100,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/

INSERT INTO PTO_PASSOSUSOFREQ (
    NRORG,
    NRPASSOSUSOFREQ,
    DSPASSOSUSOFREQ,
    OBSERVACAO,
    DSREDIRECT,
    DSREDIRECTSECTION,
    NRORDEMUSOFREQ,
    IDATIVO,
    NRORGINCLUSAO,
    DTINCLUSAO,
    CDOPERINCLUSAO,
    NRORGULTATU,
    DTULTATU,
    CDOPERULTATU
) VALUES (
    0,
    11,
    'Exportação para a Folha de Pagamento',
    'Possibilita a abertura da tela destinada à visualização do lote de frequência gerado e exportação dos respectivos lançamentos de frequência para a folha mensal.',
    'hcm01020_fpa_loteLancEvento',
    NULL,
    110,
    'S',
    1,
    SYSDATE,
    1,
    NULL,
    NULL,
    NULL
)
/
