module.exports = function(grunt) {

    var directoryNameRoutes = 'routes';
    var directoryNameConfig = 'config';
    var directoryNameServices = 'services';

    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-jshint');

    var frontendModules = [
        'assets/js/**/*'
    ];

    var backendModules = [
        '../backend/vendor/teknisa/**/*',
        '../backend/src/HCM/**/datasources/**/*',
        '../backend/config/routes/*',
        '../backend/config/services/*'
    ];

    var routes = [
        '../backend/config/routes/*'
    ];

    var dependenciesPath = [
        
        "bower_components/zhuaclfrontend/assets/dist/main.js",
        'bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js',
        "bower_components/zh-charts/dist/zh-charts.js",
        "bower_components/zh-image-uploader/dist/zh-image-uploader.js",
        "bower_components/zh-rich-text/dist/zh-rich-text.js",
        "bower_components/zh-cache-bypass/src/CacheBypass.js",
        "bower_components/zh-reports/dist/zhReports.js",
        "bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js",
        'bower_components/zh-interval-component/dist/zh-interval-component.js',
        
        'bower_components/zh-preferences/dist/preferences.js',
        'bower_components/zh-filter-preferences/dist/filter-preferences.js',
        'bower_components/zh-layout-preferences/dist/layout-preferences.js',
        'bower_components/zh-view-preferences/dist/view-preferences.js',
        'bower_components/zh-grid-configuration/dist/zh-grid-configuration.js',
        
        "bower_components/zhulog/dist/main.js",
        "bower_components/tek-libraries/dist/main.js"
    ];
    
    var cssDependenciesPath = [
        "bower_components/zh-grid-navigation/dist/style.css",
        "bower_components/zh-charts/dist/zh-charts.css",
        "bower_components/zh-rich-text/dist/zh-rich-text.css",
        "bower_components/zh-image-uploader/dist/zh-image-uploader.css",
        "bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css",
        "bower_components/zh-preferences/dist/style/style.css",
        "bower_components/zh-preferences/dist/style/colors/styles-inflightor.css",
        "bower_components/zh-reports/dist/style/colors/styles-inflightor.css"
    ];

    grunt.registerTask("createMapping", "createMapping", function() {

        function createDataSourcesMapping() {

            var dataSourcesMapping = {};

            grunt.config.get('datasources').forEach(function(dataSource) {

                var parts = dataSource.split("/");
                parts.shift();
                parts.shift();
                var filename = parts.pop();
                var path = parts.join('/');
                filename = filename.replace(/\.[^/.]+$/, "");
                dataSourcesMapping[filename] = '../' + path;

            });

            var dest = '../backend/config/dataSourcesMapping.json';
            grunt.file.write(dest, JSON.stringify(dataSourcesMapping, null, 2));

        }

        function createServicesMapping() {

            var servicesMapping = [];

            grunt.config.get('config').forEach(function(file) {
                var parts = file.split("/");
                parts.shift();
                parts.shift();
                var path = parts.join('/');
                servicesMapping.push('/' + path);
            });

            var dest = '../backend/config/servicesMapping.json';
            grunt.file.write(dest, JSON.stringify(servicesMapping, null, 2));

        }

        function createRoutesMapping() {

            var auditRoutesMapping = [];
            var logRoutesMapping = [];

            grunt.config.get('audit').forEach(function(file) {
                var routes = grunt.file.readJSON(file);
                routes.forEach(function(route) {
                    if (route.persistAudit == true) {
                        auditRoutesMapping.push(route.uri);
                    }
                    if (route.persistLog == true) {
                        logRoutesMapping.push(route.uri);
                    }
                });
            });

            var destAudit = '../backend/config/auditRoutesMapping.json';
            grunt.file.write(destAudit, JSON.stringify(auditRoutesMapping, null, 2));

            var destLog = '../backend/config/logRoutesMapping.json';
            grunt.file.write(destLog, JSON.stringify(logRoutesMapping, null, 2));

        }

        createDataSourcesMapping();
        createServicesMapping();
        createRoutesMapping();
    });

    grunt.registerTask("prepare-backend", "prepare-backend", function() {

        // temp
        var srcFiles = {
            config: [],
            datasources: []
        };

        var directories = ['config', 'datasources'];

        grunt.file.expand({
            filter: 'isFile'
        }, backendModules).forEach(function(filePath) {

            var parts = filePath.split("/");
            var filename = parts.pop();
            var directory = parts.pop();

            if (directory == directoryNameRoutes || directory == directoryNameServices)
                directory = directoryNameConfig;

            if (directories.indexOf(directory) >= 0) {
                var extension = filename.split('.').pop().toUpperCase();
                if (filename !== 'composer.json' && filename !== 'config.json' && filename !== 'config_saas.json' && filename !== 'config_saas_https.json') {
                    if (extension === 'JSON' || extension === 'XML') {
                        srcFiles[directory].push(filePath);
                    };
                }
            }

        });

        for (var key = 1 in srcFiles.config) {
            var gerPath = 'libraries/config/';
            if (srcFiles.config[key].indexOf(gerPath) != -1) {
                delete srcFiles.config[key];
            }
        };

        grunt.config('config', srcFiles.config);
        grunt.config('datasources', srcFiles.datasources);
    });



    grunt.registerTask("prepare-routes", "prepare-routes", function() {

        // temp
        var srcFiles = {
            audit: []
        };

        var directories = ['routes'];

        grunt.file.expand({
            filter: 'isFile'
        }, routes).forEach(function(filePath) {

            var parts = filePath.split("/");
            var filename = parts.pop();
            var directory = parts.pop();

            if (directories.indexOf(directory) >= 0) {
                var extension = filename.split('.').pop().toUpperCase();
                if (extension === 'JSON') {
                    srcFiles['audit'].push(filePath);
                };
            }

        });

        grunt.config('audit', srcFiles.audit);
    });

    grunt.registerTask("prepare-frontend", "prepare-frontend", function() {

        // temp
        var srcFiles = {
            repositories: [],
            containers: [],
            controllers: [],
            services: [],
            utilities: [],
            decorators: []
        };

        var directories = ['repositories', 'containers', 'controllers', 'services', 'utilities', 'decorators'];

        grunt.file.expand({
            filter: 'isFile'
        }, frontendModules).forEach(function(filePath) {

            var parts = filePath.split("/");
            var filename = parts.pop();
            var directory = parts.pop();
            if (directories.indexOf(directory) >= 0) {
                var extension = filename.split('.').pop().toUpperCase();
                if (extension === 'JS') {
                    srcFiles[directory].push(filePath);
                };
            }

        });

        var mapping = {
            containers: srcFiles.containers,
            utilities: srcFiles.utilities,
            repositories: srcFiles.repositories,
            services: srcFiles.services,
            controllers: srcFiles.controllers,
            decorators: srcFiles.decorators
        };

        var dest = './mapping.json';

        grunt.file.write(dest, JSON.stringify(mapping, null, 2));
    });

    grunt.registerTask("prepare-watch", "prepare-watch", function() {

        var mapping = grunt.file.readJSON('./mapping.json');

        var watch = {
            pivotal: {
                files: [
                    mapping.utilities,
                    mapping.repositories,
                    mapping.services,
                    mapping.controllers,
                    mapping.decorators
                ],
                tasks: ['prepare-jshint', 'prepare-concat']
            }
        };

        grunt.config.set('watch', watch);

        grunt.task.run('watch');
    });

    grunt.registerTask("prepare-concat", "prepare-concat", function() {

        var mapping = grunt.file.readJSON('./mapping.json');
        var concat = {
            hcm: {
                src: [
                    mapping.utilities,
                    mapping.repositories,
                    mapping.services,
                    mapping.controllers,
                    mapping.decorators
                ],
                dest: 'assets/js/app.js',
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                }
            },
            vendor: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: [dependenciesPath],
                dest: './assets/js/vendor.js'
            },
            vendorCss: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '/* FILE: ' + filepath + '*/\n' + src;
                    }
                },
                src: [cssDependenciesPath],
                dest: './assets/css/vendor.css'
            },
            buildAclStruct: {
                options: {
                    menuConfigFile: "./menuConfig.json"
                }
            }
        };

        grunt.config.set('concat', concat);

        grunt.task.run('concat');
    });

    grunt.registerTask("prepare-jshint", "prepare-jshint", function() {

        var mapping = grunt.file.readJSON('./mapping.json');

        var jshint = {
            all: [
                mapping.utilities,
                mapping.repositories,
                mapping.services,
                mapping.controllers,
                mapping.decorators
            ]
        };

        grunt.config.set('jshint', jshint);

        grunt.task.run('jshint');
    });

    grunt.registerTask("zhIdRemovator", "Remove ids for json items like widgets and actions", function(arg1) {
        var jsonPath = "../mobile/assets/json/containers";
        var fileName = arg1;
        if (typeof fileName == "undefined") {
            grunt.log.error("Invalid Argument. The file name must be informed.");
        }
        var filePath = jsonPath + "/" + fileName;
        var fileObject = grunt.file.readJSON(filePath);
        var countId = 0;

        function isObject(value) {
            if (Object.prototype.toString.call(value) === '[object Object]' || Object.prototype.toString.call(value) === '[object Array]') {
                return true
            }
        }

        function removeIdFromObject(object) {
            if (object['id']) {
                grunt.log.writeln("remove Id from: " + object['name']);
                delete object.id;
                countId++;
            }
        }

        function objectPropertiesIterator(object) {
            for (var property in object) {
                if (isObject(object[property])) {
                    removeIdFromObject(object[property]);
                    objectPropertiesIterator(object[property]);
                } else if (Array.isArray(object[property])) {
                    arrayIterator(object[property]);
                }
            }
        }

        function arrayIterator(array) {
            array.forEach(function(item) {
                if (isObject(item)) {
                    removeIdFromObject(item);
                    objectPropertiesIterator(item);
                } else if (Array.isArray(item)) {
                    arrayIterator(item);
                }
            });
        }

        if (isObject(fileObject)) {
            removeIdFromObject(fileObject);
            objectPropertiesIterator(fileObject);
            grunt.file.write(filePath, JSON.stringify(fileObject, null, 4));
            grunt.log.writeln(countId + " id's removed from " + fileName);
            grunt.log.writeln("File rewrited");
        } else {
            grunt.log.error("Invalid File. The file could not be found.");
        }
    });

    grunt.initConfig({
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/js/environment.min.js': 'assets/js/environment.js',
                    'config/Config.min.js': 'config/Config.js',
                    'assets/js/engine.min.js': 'assets/js/engine.js'
                }
            }
        },
        cssmin: {
            dist: {
                files: {
                    'assets/css/app.min.css': 'assets/css/app.css'
                }
            }
        },
        zhIdGenerator: {
            options: {
                jsonPath: ["../mobile/assets/json"],
                menuPath: ["../mobile/assets/json/menu.json"]
            }
        },
        zhTestBuilder: {
            options: {
                jsonPath: ["../mobile/assets/json"],
                buildPath: "tests/functional/specs/testBuilder"
            }
        },
        buildAclStruct: {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
            devCloud9: {
                configFile: "./config/devCloud9Environment.json"
            },
            development: {
                configFile: "./config/devEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            srcFolders: ["../backend/config/", "../backend/query/", "../backend/src/Reports/Birt", "./assets/js", "./assets/json", "./config", "./", "./assets", "../version/"]
        }
    });
    
    grunt.loadNpmTasks('grunt-zh-id-generator');

    grunt.loadNpmTasks('grunt-zhuacl-structure');

    grunt.loadNpmTasks('grunt-zh-environment-builder');
    
    grunt.loadNpmTasks('zeedhi-grunt-test-builder');
    
    grunt.loadNpmTasks('grunt-contrib-uglify');
    
    grunt.loadNpmTasks('grunt-contrib-cssmin');

   /* grunt.registerTask('build-dev', [
        'prepare-frontend',
        'prepare-backend',
        'prepare-routes',
        'createMapping',
        'prepare-jshint',
        'prepare-concat',
        'prepare-watch'
    ]);*/

    grunt.registerTask('init_project', [
        'prepare-frontend',
        'prepare-backend',
        'prepare-routes',
        'createMapping',
        'prepare-jshint',
        'prepare-concat',
        'uglify',
        'cssmin'
    ]);
};
